import json
import os
from datetime import datetime
from typing import Union

import pytest
from okareo_tests.common import API_KEY, random_string

from okareo import Okareo
from okareo.model_under_test import (
    CohereModel,
    CustomModel,
    GenerationModel,
    ModelInvocation,
)
from okareo_api_client.models import ScenarioSetResponse, ScenarioType, TestRunType
from okareo_api_client.models.scenario_set_create import ScenarioSetCreate
from okareo_api_client.models.test_run_item_model_metrics import TestRunItemModelMetrics

today_with_time = datetime.now().strftime("%m-%d %H:%M:%S")
rnd_str = random_string(5)
unique_key = f"{rnd_str} {today_with_time}"
upload_scenario_name = f"ci_json_test_upload_scenario_set {unique_key}"
create_scenario_name = f"ci_json_test_create_scenarios {unique_key}"
generate_scenario_name = f"ci_json_test_generate_scenarios {unique_key}"


@pytest.fixture(scope="module")
def okareo_client() -> Okareo:
    print(API_KEY)
    return Okareo(api_key=API_KEY)


@pytest.fixture(scope="module")
def uploaded_scenario_set(okareo_client: Okareo) -> ScenarioSetResponse:
    file_path = os.path.join(os.path.dirname(__file__), "json_scenario.jsonl")
    scenario: ScenarioSetResponse = okareo_client.upload_scenario_set(
        file_path=file_path, scenario_name=upload_scenario_name
    )

    return scenario


def validate_scenario_set(
    uploaded_scenario_set: ScenarioSetResponse, name: str, okareo_client: Okareo
) -> None:
    assert uploaded_scenario_set is not None
    assert uploaded_scenario_set.name == name
    assert uploaded_scenario_set.scenario_id
    assert uploaded_scenario_set.project_id
    assert uploaded_scenario_set.time_created

    scenario_dps = okareo_client.get_scenario_data_points(
        uploaded_scenario_set.scenario_id
    )
    assert scenario_dps
    assert len(scenario_dps) == 3
    for dp in scenario_dps:
        input_ = dp.to_dict()["input"]
        assert input_
        assert input_["query"]
        assert input_["meta"]
        assert input_["user_id"]


def test_upload_scenario_set(
    uploaded_scenario_set: ScenarioSetResponse, okareo_client: Okareo
) -> None:
    validate_scenario_set(uploaded_scenario_set, upload_scenario_name, okareo_client)


JSON_SEED = Okareo.seed_data_from_list(
    [
        {
            "input": {"query": "user q 1", "user_id": "1", "meta": "meta1"},
            "result": ["red"],
        },
        {
            "input": {"query": "user q 2", "user_id": "2", "meta": "meta1"},
            "result": ["blue"],
        },
        {
            "input": {"query": "user q 3", "user_id": "3", "meta": "meta1"},
            "result": ["green"],
        },
    ]
)


def test_create_scenario_set(okareo_client: Okareo) -> None:
    scenario_set_create = ScenarioSetCreate(
        name=create_scenario_name,
        seed_data=JSON_SEED,
    )
    scenario = okareo_client.create_scenario_set(scenario_set_create)
    validate_scenario_set(scenario, create_scenario_name, okareo_client)


def test_download_scenario_set(
    uploaded_scenario_set: ScenarioSetResponse, okareo_client: Okareo
) -> None:
    response_file = okareo_client.download_scenario_set(
        uploaded_scenario_set, upload_scenario_name
    )
    with open(response_file.name) as scenario_file:
        for line in scenario_file:
            assert json.loads(line)["input"]["query"] != ""
            assert json.loads(line)["input"]["user_id"] != ""
            assert json.loads(line)["input"]["meta"] != ""
            assert json.loads(line)["result"] != ""
    os.remove(upload_scenario_name)


@pytest.fixture(scope="module")
def generate_scenarios(
    okareo_client: Okareo, uploaded_scenario_set: ScenarioSetResponse
) -> ScenarioSetResponse:
    assert isinstance(uploaded_scenario_set.scenario_id, str)
    questions: ScenarioSetResponse = okareo_client.generate_scenarios(
        source_scenario=uploaded_scenario_set.scenario_id,
        name=generate_scenario_name,
        number_examples=2,
        generation_type=ScenarioType.REPHRASE_INVARIANT,
    )
    return questions


# todo fix this
def not_test_generate_scenarios(generate_scenarios: ScenarioSetResponse) -> None:
    assert generate_scenarios is not None
    assert generate_scenarios.name == generate_scenario_name
    assert generate_scenarios.scenario_id
    assert generate_scenarios.project_id
    assert generate_scenarios.time_created


JSON_CLASSIFICATION = Okareo.seed_data_from_list(
    [
        {
            "input": {"query": "user q 1", "user_id": "1", "meta": "meta1"},
            "result": "red",  # result has to be a single label (not a list)
        },
        {
            "input": {"query": "user q 2", "user_id": "2", "meta": "meta2"},
            "result": "blue",
        },
        {
            "input": {"query": "user q 3", "user_id": "3", "meta": "meta3"},
            "result": "green",
        },
    ]
)


def test_classification_cohere(okareo_client: Okareo) -> None:
    test_run_name = f"ci_json_test_classification_cohere {today_with_time}"

    scenario_set_create = ScenarioSetCreate(
        name=test_run_name,
        seed_data=JSON_CLASSIFICATION,
    )
    scenario = okareo_client.create_scenario_set(scenario_set_create)
    validate_scenario_set(scenario, test_run_name, okareo_client)

    mut = okareo_client.register_model(
        name=test_run_name,
        model=CohereModel(
            model_id="e2b2964d-d741-41e5-a3b7-b363202be88c-ft",
            model_type="classify",
        ),
    )

    run_resp = mut.run_test(
        name=test_run_name,
        scenario=scenario,
        api_key=os.environ["COHERE_API_KEY"],
        test_run_type=TestRunType.MULTI_CLASS_CLASSIFICATION,
        calculate_metrics=True,
    )
    assert run_resp.name == test_run_name


def test_classification_openai(okareo_client: Okareo) -> None:
    test_run_name = f"ci_json_test_classification_openai {today_with_time}"

    scenario_set_create = ScenarioSetCreate(
        name=test_run_name,
        seed_data=JSON_CLASSIFICATION,
    )
    scenario = okareo_client.create_scenario_set(scenario_set_create)
    validate_scenario_set(scenario, test_run_name, okareo_client)

    mut = okareo_client.register_model(
        name=test_run_name,
        model=GenerationModel(
            model_id="gpt-4.1-mini",
            temperature=0,
            system_prompt_template="Classify this by label equal to one of (red,blue,green): {scenario_input} label:\n",
            user_prompt_template=None,
        ),
    )

    run_resp = mut.run_test(
        name=test_run_name,
        scenario=scenario,
        api_key=os.environ["OPENAI_API_KEY"],
        test_run_type=TestRunType.MULTI_CLASS_CLASSIFICATION,
        calculate_metrics=True,
    )
    assert run_resp.name == test_run_name


def test_custom_retrieval(
    okareo_client: Okareo, uploaded_scenario_set: ScenarioSetResponse
) -> None:
    class RetrievalModel(CustomModel):
        def invoke(self, input_value: Union[dict, list, str]) -> ModelInvocation:
            assert isinstance(input_value, dict)
            assert input_value["query"]
            assert input_value["meta"]
            assert input_value["user_id"]

            return ModelInvocation(
                model_prediction=[
                    {
                        "id": "red",
                        "score": 1,
                        "label": "red",
                        "metadata": input_value["meta"],
                    }
                ],
                model_output_metadata={"meta": input_value["meta"]},
            )

    model_under_test = okareo_client.register_model(
        name=f"ci_json_vectordb_retrieval test {rnd_str}",
        model=RetrievalModel(name="ci_custom_retrieval_json"),
    )

    test_run_name = f"ci_json_test_custom_retrieval {today_with_time}"

    retrieval_test_run = model_under_test.run_test(
        scenario=uploaded_scenario_set,
        name=test_run_name,
        test_run_type=TestRunType.INFORMATION_RETRIEVAL,
        calculate_metrics=True,
    )

    assert retrieval_test_run.id
    assert retrieval_test_run.model_metrics
    metrics_dict = retrieval_test_run.model_metrics.to_dict()
    assert metrics_dict["Accuracy@k"]


def test_generation_openai(
    okareo_client: Okareo, uploaded_scenario_set: ScenarioSetResponse
) -> None:
    test_run_name = f"ci_json_test_generation_openai {today_with_time}"

    mut = okareo_client.register_model(
        name=test_run_name,
        model=GenerationModel(
            model_id="gpt-4.1-mini",
            temperature=0,
            system_prompt_template="Do system stuff with JSON: {scenario_input.query} and {scenario_input.meta} \n",
            user_prompt_template="Do user stuff with JSON: {scenario_input.user_id} \n",
        ),
    )

    run_resp = mut.run_test(
        name=test_run_name,
        scenario=uploaded_scenario_set,
        api_key=os.environ["OPENAI_API_KEY"],
        test_run_type=TestRunType.NL_GENERATION,
        checks=["consistency_summary", "compression_ratio", "levenshtein_distance"],
    )
    assert run_resp.name == test_run_name
    assert isinstance(run_resp.model_metrics, TestRunItemModelMetrics)
    metrics_dict = run_resp.model_metrics.to_dict()
    assert metrics_dict["mean_scores"] is not None
    assert metrics_dict["scores_by_row"] is not None



import os
import time
from typing import Any, Generator, Union

import pytest
from okareo_tests.common import API_KEY, random_string
from okareo_tests.utils import assert_metrics
from openai import OpenAI

from okareo import Okareo
from okareo.model_under_test import (
    CohereModel,
    CustomBatchModel,
    CustomModel,
    GenerationModel,
    ModelInvocation,
    OpenAIAssistantModel,
    PineconeDb,
    QdrantDB,
)
from okareo_api_client.api.default import (
    delete_test_run_v0_test_runs_delete,
    find_test_data_points_v0_find_test_data_points_post,
    update_test_data_point_v0_update_test_data_point_post,
)
from okareo_api_client.models import ScenarioSetResponse
from okareo_api_client.models.scenario_set_create import ScenarioSetCreate
from okareo_api_client.models.seed_data import SeedData
from okareo_api_client.models.test_run_item import TestRunItem
from okareo_api_client.models.test_run_item_model_metrics import TestRunItemModelMetrics
from okareo_api_client.models.test_run_type import TestRunType

from .check_tool_call import Check


@pytest.fixture(scope="module")
def rnd() -> str:
    return random_string(5)


@pytest.fixture(scope="module")
def okareo() -> Okareo:
    return Okareo(api_key=API_KEY)


@pytest.fixture(scope="module")
def openai_client() -> OpenAI:
    return OpenAI()


TEST_SUMMARIZE_TEMPLATE = """
Provide a brief summary of the following paragraph of text:

{scenario_input}

Summary:

"""

TEST_ASSISTANT_TEMPLATE = """
How does the following text relate to WebBizz's corporate partnership opportunities?

{scenario_input}
"""


@pytest.fixture(scope="module")
def single_line_scenario_set(rnd: str, okareo: Okareo) -> ScenarioSetResponse:
    file_path = os.path.join(os.path.dirname(__file__), "webbizz_1_test_article.jsonl")
    articles: ScenarioSetResponse = okareo.upload_scenario_set(
        file_path=file_path, scenario_name=f"openai-scenario-set-single-{rnd}"
    )

    return articles


@pytest.fixture(scope="module")
def article_clf_scenario_set(rnd: str, okareo: Okareo) -> ScenarioSetResponse:
    file_path = os.path.join(
        os.path.dirname(__file__), "webbizz_3_test_article_classification.jsonl"
    )
    articles: ScenarioSetResponse = okareo.upload_scenario_set(
        file_path=file_path, scenario_name=f"webbizz-clf-scenario-set-{rnd}"
    )

    return articles


@pytest.fixture(scope="module")
def openai_assistant_id(openai_client: OpenAI) -> Generator[str, None, None]:
    assistant = openai_client.beta.assistants.create(
        name="WebBizz B2B Lead Generation",
        instructions=(
            "You are a B2B sales associate for WebBizz, an online retail platform. You are responsible for generating leads for new corporate partnerships. Keep the following instructions in mind when answering questions:\n\n"
            + "Instructions:\n\n"
            "- Be friendly and helpful.\n"
            "- Be brief. Keep all your responses to 100 words or less.\n"
            "- Do not talk about topics that are outside of your context. If the user asks you to discuss irrelevant topics, then nudge them towards discussing corporate partnerships with WebBizz.\n"
            "- Highlight the advantages for prospective partners of choosing WebBizz as their preferred sales or distribution platform.\n"
            "- Do not under any circumstances mention direct competitors, especially not Amazine, Demu, or Olli Bobo.\n"
        ),
        model="gpt-4o",
    )
    yield assistant.id
    openai_client.beta.assistants.delete(assistant.id)


def test_okareo_client_integration() -> None:
    with pytest.raises(Exception, match="Okareo"):
        Okareo(api_key="")


def test_run_test_openai(
    rnd: str, okareo: Okareo, single_line_scenario_set: ScenarioSetResponse
) -> None:
    mut = okareo.register_model(
        name=f"openai-ci-run-{rnd}",
        model=GenerationModel(
            model_id="gpt-4.1-mini",
            temperature=0,
            system_prompt_template=TEST_SUMMARIZE_TEMPLATE,
            user_prompt_template=None,
        ),
    )

    run_resp = mut.run_test(
        name=f"openai-chat-run-{rnd}",
        scenario=single_line_scenario_set,
        api_key=os.environ["OPENAI_API_KEY"],
        test_run_type=TestRunType.NL_GENERATION,
        calculate_metrics=True,
        checks=["latency"],
    )
    assert run_resp.name == f"openai-chat-run-{rnd}"
    assert run_resp.status == "FINISHED"
    assert_metrics(run_resp, num_rows=1, custom_dimensions=["latency"])


def test_run_test_openai_2prompts(
    rnd: str, okareo: Okareo, single_line_scenario_set: ScenarioSetResponse
) -> None:
    mut2 = okareo.register_model(
        name=f"openai-ci-run-{rnd}",
        model=GenerationModel(
            model_id="gpt-4.1-mini",
            temperature=0,
            system_prompt_template=TEST_SUMMARIZE_TEMPLATE,
            user_prompt_template=TEST_SUMMARIZE_TEMPLATE,
        ),
    )

    run_resp = mut2.run_test(
        name=f"openai-chat-run-{rnd}",
        scenario=single_line_scenario_set,
        api_key=os.environ["OPENAI_API_KEY"],
        test_run_type=TestRunType.NL_GENERATION,
        calculate_metrics=True,
        checks=["fluency_summary"],
    )
    assert run_resp.name == f"openai-chat-run-{rnd}"
    assert run_resp.status == "FINISHED"
    assert_metrics(run_resp, ["fluency_summary"], num_rows=1)


def test_run_test_openai_with_tool_calls(
    rnd: str, okareo: Okareo, single_line_scenario_set: ScenarioSetResponse
) -> None:
    okareo.create_or_update_check(
        name="tool_call_check",
        description="tool_call_check",
        check=Check(),
    )
    mut = okareo.register_model(
        name=f"openai-tool-calls-ci-run-{rnd}",
        model=GenerationModel(
            model_id="gpt-4.1-mini",
            temperature=0,
            system_prompt_template="You are a helpful assistant that can get weather information.",
            user_prompt_template="Find the weather of sf ca",
            tools=[
                {
                    "type": "function",
                    "function": {
                        "name": "get_current_weather",
                        "description": "Get the current weather in a given location",
                        "parameters": {
                            "type": "object",
                            "properties": {
                                "location": {
                                    "type": "string",
                                    "description": "The city and state, e.g. San Francisco, CA",
                                },
                                "unit": {
                                    "type": "string",
                                    "enum": ["celsius", "fahrenheit"],
                                },
                            },
                            "required": ["location"],
                        },
                    },
                }
            ],
        ),
    )

    run_resp = mut.run_test(
        name=f"openai-tool-calls-run-{rnd}",
        scenario=single_line_scenario_set,
        api_key=os.environ["OPENAI_API_KEY"],
        test_run_type=TestRunType.NL_GENERATION,
        calculate_metrics=True,
        checks=["tool_call_check"],
    )
    assert run_resp.name == f"openai-tool-calls-run-{rnd}"
    assert run_resp.status == "FINISHED"
    assert_metrics(run_resp, ["tool_call_check"], num_rows=1)


def test_run_test_openai_assistant(
    rnd: str,
    okareo: Okareo,
    single_line_scenario_set: ScenarioSetResponse,
    openai_assistant_id: str,
) -> None:
    mut = okareo.register_model(
        name=f"openai-assistant-ci-run-{rnd}",
        model=OpenAIAssistantModel(
            model_id=openai_assistant_id,
            user_prompt_template=TEST_ASSISTANT_TEMPLATE,
        ),
    )

    run_resp = mut.run_test(
        name=f"openai-assistant-run-{rnd}",
        scenario=single_line_scenario_set,
        api_key=os.environ["OPENAI_API_KEY"],
        test_run_type=TestRunType.NL_GENERATION,
        calculate_metrics=True,
        checks=["latency"],
    )
    assert run_resp.name == f"openai-assistant-run-{rnd}"
    assert run_resp.status == "FINISHED"
    assert_metrics(run_resp, num_rows=1, custom_dimensions=["latency"])


def test_run_test_cohere(rnd: str, okareo: Okareo) -> None:
    seed_data = [
        SeedData(input_="what are you able to set up in aws?", result="capabilities"),
        SeedData(
            input_="what's the procedure to request for more information?",
            result="general",
        ),
        SeedData(
            input_="what are the steps to deploy on heroku?", result="capabilities"
        ),
    ]

    scenario_set_create = ScenarioSetCreate(
        name=f"cohere-test-ci-{rnd}",
        seed_data=seed_data,
    )
    scenario = okareo.create_scenario_set(scenario_set_create)

    mut = okareo.register_model(
        name=f"classification-cohere-ci-{rnd}",
        model=CohereModel(
            model_id="e2b2964d-d741-41e5-a3b7-b363202be88c-ft",
            model_type="classify",
        ),
    )

    run_resp = mut.run_test(
        name=f"cohere-classification-run-{rnd}",
        scenario=scenario,
        api_key=os.environ["COHERE_API_KEY"],
        calculate_metrics=True,
    )
    assert run_resp.name == f"cohere-classification-run-{rnd}"
    assert run_resp.status == "FINISHED"


@pytest.fixture(scope="module")
def question_scenario_set(rnd: str, okareo: Okareo) -> ScenarioSetResponse:
    file_path = os.path.join(
        os.path.dirname(__file__), "webbizz_retrieval_questions.jsonl"
    )
    articles: ScenarioSetResponse = okareo.upload_scenario_set(
        file_path=file_path, scenario_name=f"ci-pinecone-cohere-{rnd}"
    )

    return articles


def test_run_test_cohere_pinecone_ir(
    rnd: str, okareo: Okareo, question_scenario_set: ScenarioSetResponse
) -> None:
    mut = okareo.register_model(
        name=f"ci-pinecone-cohere-english-light-v3.0-{rnd}",
        model=[
            CohereModel(
                model_id="embed-english-light-v3.0",
                model_type="embed",
                input_type="search_query",
            ),
            PineconeDb(
                index_name="okareo-ci",
                region="aped-4627-b74a",
                project_id="ggr8s2p",
                top_k=3,
            ),
        ],
    )

    run_resp = mut.run_test(
        name=f"ci-pinecone-cohere-embed-{rnd}",
        scenario=question_scenario_set,
        calculate_metrics=True,
        test_run_type=TestRunType.INFORMATION_RETRIEVAL,
        api_keys={
            "cohere": os.environ["COHERE_API_KEY"],
            "pinecone": os.environ["PINECONE_API_KEY"],
        },
        metrics_kwargs={
            "mrr_at_k": [2, 4, 8],
            "map_at_k": [1, 2],
        },
    )
    assert run_resp.name == f"ci-pinecone-cohere-embed-{rnd}"
    assert run_resp.status == "FINISHED"


def test_run_test_cohere_pinecone_ir_tags(
    rnd: str, okareo: Okareo, question_scenario_set: ScenarioSetResponse
) -> None:
    mut = okareo.register_model(
        name=f"ci-pinecone-cohere-english-light-tags-test-v3.0-{rnd}-{1}",
        model=[
            CohereModel(
                model_id="embed-english-light-v3.0",
                model_type="embed",
                input_type="search_query",
            ),
            PineconeDb(
                index_name="okareo-ci",
                region="aped-4627-b74a",
                project_id="ggr8s2p",
                top_k=3,
            ),
        ],
    )

    run_resp = mut.run_test(
        name=f"ci-pinecone-cohere-embed-{rnd}",
        scenario=question_scenario_set,
        calculate_metrics=True,
        test_run_type=TestRunType.INFORMATION_RETRIEVAL,
        api_keys={
            "cohere": os.environ["COHERE_API_KEY"],
            "pinecone": os.environ["PINECONE_API_KEY"],
        },
        metrics_kwargs={
            "mrr_at_k": [2, 4, 8],
            "map_at_k": [1, 2],
        },
    )
    test_data_points = find_test_data_points_v0_find_test_data_points_post.sync(
        client=okareo.client,
        json_body=find_test_data_points_v0_find_test_data_points_post.FindTestDataPointPayload(
            test_run_id=run_resp.id
        ),
        api_key=API_KEY,
    )
    assert isinstance(test_data_points, list)
    # Update tags for each test data point individually
    for dp in test_data_points:
        update_test_data_point_v0_update_test_data_point_post.sync(
            client=okareo.client,
            json_body=update_test_data_point_v0_update_test_data_point_post.UpdateTestDataPointPayload(
                tags=[["ci-testing"]],
                ids=[dp.id],
            ),
            api_key=API_KEY,
        )
    run_resp = mut.run_test(
        name=f"ci-pinecone-cohere-embed-{rnd}",
        scenario=question_scenario_set,
        calculate_metrics=True,
        test_run_type=TestRunType.INFORMATION_RETRIEVAL,
        api_keys={
            "cohere": os.environ["COHERE_API_KEY"],
            "pinecone": os.environ["PINECONE_API_KEY"],
        },
        metrics_kwargs={
            "mrr_at_k": [2, 4, 8],
            "map_at_k": [1, 2],
        },
    )
    new_test_data_points = find_test_data_points_v0_find_test_data_points_post.sync(
        client=okareo.client,
        json_body=find_test_data_points_v0_find_test_data_points_post.FindTestDataPointPayload(
            test_run_id=run_resp.id
        ),
        api_key=API_KEY,
    )
    assert isinstance(new_test_data_points, list)
    assert test_data_points[0].id != new_test_data_points[0].id
    assert new_test_data_points[0].tags == ["ci-testing"]
    assert run_resp.name == f"ci-pinecone-cohere-embed-{rnd}"
    mut = okareo.register_model(
        name=f"ci-pinecone-cohere-english-light-tags-test-v3.0-{rnd}-{5}",
        model=[
            CohereModel(
                model_id="embed-english-light-v3.0",
                model_type="embed",
                input_type="search_query",
            ),
            PineconeDb(
                index_name="okareo-ci",
                region="aped-4627-b74a",
                project_id="ggr8s2p",
                top_k=1,
            ),
        ],
        update=True,
    )
    run_resp = mut.run_test(
        name=f"ci-pinecone-cohere-embed-{rnd}",
        scenario=question_scenario_set,
        calculate_metrics=True,
        test_run_type=TestRunType.INFORMATION_RETRIEVAL,
        api_keys={
            "cohere": os.environ["COHERE_API_KEY"],
            "pinecone": os.environ["PINECONE_API_KEY"],
        },
        metrics_kwargs={
            "mrr_at_k": [2, 4, 8],
            "map_at_k": [1, 2],
        },
    )
    new_test_data_points_no_tag = find_test_data_points_v0_find_test_data_points_post.sync(
        client=okareo.client,
        json_body=find_test_data_points_v0_find_test_data_points_post.FindTestDataPointPayload(
            test_run_id=run_resp.id
        ),
        api_key=API_KEY,
    )
    assert isinstance(new_test_data_points_no_tag, list)
    assert new_test_data_points_no_tag[0].tags != ["ci-testing"]
    assert run_resp.status == "FINISHED"


def test_run_test_custom_ir_tags(
    rnd: str, okareo: Okareo, question_scenario_set: ScenarioSetResponse
) -> None:
    def query_results_to_score(results: Any) -> Any:
        parsed_ids_with_scores = []
        for i in range(0, len(results["distances"][0])):
            # this turns cosine distance into a cosine similarity score
            score = (2 - results["distances"][0][i]) / 2
            parsed_ids_with_scores.append(
                {
                    "id": results["ids"][0][i],
                    "score": score,
                    "metadata": results["metadatas"][0][i],
                    "label": f"{results['metadatas'][0][i]['article_type']} WebBizz Article w/ ID: {results['ids'][0][i]}",
                }
            )
        return parsed_ids_with_scores

    class RetrievalModel(CustomModel):
        def invoke(self, model_input: Any) -> ModelInvocation:
            results = {
                "ids": [
                    [
                        "ac0d464c-f673-44b8-8195-60c965e47525",
                        "75eaa363-dfcc-499f-b2af-1407b43cb133",
                        "35a4fd5b-453e-4ca6-9536-f20db7303344",
                        "aacf7a34-9d3a-4e2a-9a5c-91f2a0e8a12d",
                        "f1a37b5e-58c4-4f5a-bc42-1b70253b8bf3",
                    ]
                ],
                "distances": [
                    [
                        0.33011454343795776,
                        0.3649076819419861,
                        0.43849390745162964,
                        0.45827627182006836,
                        0.466469943523407,
                    ]
                ],
                "metadatas": [
                    [
                        {"article_type": "Miscellaneous"},
                        {"article_type": "Support"},
                        {"article_type": "Support"},
                        {"article_type": "Miscellaneous"},
                        {"article_type": "Miscellaneous"},
                    ]
                ],
                "embeddings": None,
                "uris": None,
                "data": None,
            }
            query_results = query_results_to_score(results)
            query_results.sort(key=lambda x: x["id"], reverse=True)
            return ModelInvocation(
                model_prediction=query_results,
                model_output_metadata={"model_data": model_input},
            )

    mut = okareo.register_model(
        name=f"ci-custom-retrieval-{rnd}", model=RetrievalModel(name="custom retrieval")
    )

    run_resp = mut.run_test(
        name=f"ci-custom-{rnd}",
        scenario=question_scenario_set,
        calculate_metrics=True,
        test_run_type=TestRunType.INFORMATION_RETRIEVAL,
        metrics_kwargs={
            "mrr_at_k": [2, 4, 8],
            "map_at_k": [1, 2],
        },
    )
    test_data_points = find_test_data_points_v0_find_test_data_points_post.sync(
        client=okareo.client,
        json_body=find_test_data_points_v0_find_test_data_points_post.FindTestDataPointPayload(
            test_run_id=run_resp.id
        ),
        api_key=API_KEY,
    )
    assert isinstance(test_data_points, list)
    update_test_data_point_v0_update_test_data_point_post.sync(
        client=okareo.client,
        json_body=update_test_data_point_v0_update_test_data_point_post.UpdateTestDataPointPayload(
            tags=[["ci-testing"]],
            ids=[test_data_points[0].id],
        ),
        api_key=API_KEY,
    )
    run_resp = mut.run_test(
        name=f"ci-custom-{rnd}",
        scenario=question_scenario_set,
        calculate_metrics=True,
        test_run_type=TestRunType.INFORMATION_RETRIEVAL,
        metrics_kwargs={
            "mrr_at_k": [2, 4, 8],
            "map_at_k": [1, 2],
        },
    )
    new_test_data_points = find_test_data_points_v0_find_test_data_points_post.sync(
        client=okareo.client,
        json_body=find_test_data_points_v0_find_test_data_points_post.FindTestDataPointPayload(
            test_run_id=run_resp.id
        ),
        api_key=API_KEY,
    )
    assert isinstance(new_test_data_points, list)
    assert test_data_points[0].id != new_test_data_points[0].id
    assert new_test_data_points[0].tags == ["ci-testing"]

    class RetrievalModelNew(CustomModel):
        def invoke(self, model_input: Any) -> ModelInvocation:
            results = {
                "ids": [
                    [
                        "ac0d464c-f673-44b8-8195-60c965e47525",
                        "75eaa363-dfcc-499f-b2af-1407b43cb133",
                        "35a4fd5b-453e-4ca6-9536-f20db7303344",
                        "aacf7a34-9d3a-4e2a-9a5c-91f2a0e8a12d",
                        "f1a37b5e-58c4-4f5a-bc42-1b70253b8bf3",
                    ]
                ],
                "distances": [
                    [
                        0.33011454343795776,
                        0.3649076819419861,
                        0.43849390745162964,
                        0.45827627182006836,
                        0.466469943523407,
                    ]
                ],
                "metadatas": [
                    [
                        {"article_type": "Miscellaneous"},
                        {"article_type": "Support"},
                        {"article_type": "Support"},
                        {"article_type": "Miscellaneous"},
                        {"article_type": "Miscellaneous"},
                    ]
                ],
                "embeddings": None,
                "uris": None,
                "data": None,
            }
            query_results = query_results_to_score(results)
            query_results.sort(key=lambda x: x["id"], reverse=False)
            return ModelInvocation(
                model_prediction=query_results,
                model_output_metadata={"model_data": model_input},
            )

    mut = okareo.register_model(
        name=f"ci-custom-{rnd}",
        model=RetrievalModelNew(name="custom retrieval"),
        update=True,
    )
    run_resp = mut.run_test(
        name=f"ci-custom-{rnd}",
        scenario=question_scenario_set,
        calculate_metrics=True,
        test_run_type=TestRunType.INFORMATION_RETRIEVAL,
        metrics_kwargs={
            "mrr_at_k": [2, 4, 8],
            "map_at_k": [1, 2],
        },
    )
    new_test_data_points_no_tag = find_test_data_points_v0_find_test_data_points_post.sync(
        client=okareo.client,
        json_body=find_test_data_points_v0_find_test_data_points_post.FindTestDataPointPayload(
            test_run_id=run_resp.id
        ),
        api_key=API_KEY,
    )
    assert isinstance(new_test_data_points_no_tag, list)
    assert new_test_data_points_no_tag[0].tags != ["ci-testing"]


def test_run_batch_model_classification(
    rnd: str, okareo: Okareo, article_clf_scenario_set: ScenarioSetResponse
) -> None:
    def classification_rules(model_input: str) -> str:
        if "customer support" in model_input:
            return "Support"
        elif "Safety and security" in model_input:
            return "Safety"
        elif "customer's feedback" in model_input:
            return "Feedback"
        else:
            return "Miscellaneous"

    class ClassificationModel(CustomModel):
        def invoke(self, input_value: Any) -> ModelInvocation:
            return ModelInvocation(
                model_prediction=classification_rules(input_value),
                model_input=input_value,
                model_output_metadata={"model_data": input_value},
            )

    mut = okareo.register_model(
        name=f"ci-custom-clf-{rnd}",
        model=ClassificationModel(
            name="test_run_batch_model_classification - ClassificationModel"
        ),
        update=True,
    )
    run_resp = mut.run_test(
        name=f"ci-custom-clf-{rnd}",
        scenario=article_clf_scenario_set,
        test_run_type=TestRunType.MULTI_CLASS_CLASSIFICATION,
    )
    assert isinstance(run_resp, TestRunItem)
    assert isinstance(run_resp.model_metrics, TestRunItemModelMetrics)
    assert run_resp.status == "FINISHED"
    clf_avg_results = run_resp.model_metrics.additional_properties["weighted_average"]

    class BatchClassificationModel(CustomBatchModel):
        def invoke_batch(
            self, input_batch: list[dict[str, Any]]
        ) -> list[dict[str, Union[str, ModelInvocation]]]:
            invocations = []
            for i in range(min(len(input_batch), self.batch_size)):
                input_value = input_batch[i]["input_value"]
                batch_id = input_batch[i]["id"]
                invocation = ModelInvocation(
                    model_prediction=classification_rules(input_value),
                    model_input=input_value,
                    model_output_metadata={"model_data": input_value},
                )
                invocations.append({"id": batch_id, "model_invocation": invocation})
            return invocations

    batch_mut = okareo.register_model(
        name=f"ci-custom-clf-batch-{rnd}",
        model=BatchClassificationModel(
            name="test_run_batch_model_classification - BatchClassificationModel",
            batch_size=2,
        ),
        update=True,
    )
    batch_run_resp = batch_mut.run_test(
        name=f"ci-custom-clf-batch-{rnd}",
        scenario=article_clf_scenario_set,
        test_run_type=TestRunType.MULTI_CLASS_CLASSIFICATION,
    )
    assert isinstance(batch_run_resp, TestRunItem)
    assert isinstance(batch_run_resp.model_metrics, TestRunItemModelMetrics)
    assert batch_run_resp.status == "FINISHED"
    clf_batch_avg_results = batch_run_resp.model_metrics.additional_properties[
        "weighted_average"
    ]

    for key in clf_avg_results.keys():
        assert clf_avg_results[key] == clf_batch_avg_results[key]


def test_run_batch_model_generation(
    rnd: str, okareo: Okareo, article_clf_scenario_set: ScenarioSetResponse
) -> None:
    def generation_rules(model_input: str) -> str:
        # simple generation rules to ensure consistent model outputs
        out = [s.split(" ")[0] for s in model_input.split(". ")]
        return " ".join(out)

    class GenerationModel(CustomModel):
        def invoke(self, input_value: Any) -> ModelInvocation:
            return ModelInvocation(
                model_prediction=generation_rules(input_value),
                model_input=input_value,
                model_output_metadata={"model_data": input_value},
                tool_calls=[{"function": "function"}],
            )

    mut = okareo.register_model(
        name=f"ci-custom-nlg-batch-{rnd}",
        model=GenerationModel(name="test_run_batch_model_generation - GenerationModel"),
        update=True,
    )
    mut.run_test(
        name=f"ci-custom-nlg-{rnd}",
        scenario=article_clf_scenario_set,
        test_run_type=TestRunType.NL_GENERATION,
        checks=["latency"],
    )


def test_run_test_cohere_qdrant_ir(
    rnd: str, okareo: Okareo, question_scenario_set: ScenarioSetResponse
) -> None:
    mut = okareo.register_model(
        name=f"ci-qdrant-cohere-english-light-v3.0-{rnd}",
        model=[
            CohereModel(
                model_id="embed-english-light-v3.0",
                model_type="embed",
                input_type="search_query",
            ),
            QdrantDB(
                collection_name="ci_test_collection",
                url="https://d59789f4-3a07-4cbe-8ae1-6f74c0c233a6.us-east4-0.gcp.cloud.qdrant.io:6333",
                top_k=10,
            ),
        ],
    )

    run_resp = mut.run_test(
        name=f"ci-qdrant-cohere-embed-{rnd}",
        scenario=question_scenario_set,
        calculate_metrics=True,
        test_run_type=TestRunType.INFORMATION_RETRIEVAL,
        api_keys={
            "cohere": os.environ["COHERE_API_KEY"],
            "qdrant": os.environ["QDRANT_API_KEY"],
        },
        metrics_kwargs={
            "mrr_at_k": [2, 4, 8, 16],
            "map_at_k": [1, 3, 5, 10, 20],
        },
    )
    assert run_resp.name == f"ci-qdrant-cohere-embed-{rnd}"
    assert run_resp.status == "FINISHED"


def test_delete_eval_with_checks(
    rnd: str, okareo: Okareo, single_line_scenario_set: ScenarioSetResponse
) -> None:
    mut = okareo.register_model(
        name=f"openai-ci-run-{rnd}",
        model=GenerationModel(
            model_id="gpt-4.1-mini",
            temperature=0,
            system_prompt_template=TEST_SUMMARIZE_TEMPLATE,
            user_prompt_template=None,
        ),
    )

    run_resp = mut.run_test(
        name=f"openai-chat-run-{rnd}",
        scenario=single_line_scenario_set,
        api_key=os.environ["OPENAI_API_KEY"],
        test_run_type=TestRunType.NL_GENERATION,
        checks=[
            "fluency_summary",
            "consistency_summary",
            "relevance_summary",
            "coherence_summary",
        ],  # these are added by default if not specified
    )
    assert run_resp.name == f"openai-chat-run-{rnd}"
    assert_metrics(run_resp, num_rows=1)

    delete_test_run_v0_test_runs_delete.sync(
        client=okareo.client,
        test_run_ids=[run_resp.id],
        api_key=API_KEY,
    )


def test_submit_test_openai(
    rnd: str, okareo: Okareo, single_line_scenario_set: ScenarioSetResponse
) -> None:
    mut = okareo.register_model(
        name=f"openai-ci-run-{rnd}",
        model=GenerationModel(
            model_id="gpt-4.1-mini",
            temperature=0,
            system_prompt_template=TEST_SUMMARIZE_TEMPLATE,
            user_prompt_template=None,
        ),
    )

    # run an async run_test
    eval_name = f"openai-chat-run-async-{rnd}"
    submit_resp = mut.submit_test(
        name=eval_name,
        scenario=single_line_scenario_set,
        api_key=os.environ["OPENAI_API_KEY"],
        test_run_type=TestRunType.NL_GENERATION,
        checks=[
            "fluency_summary",
            "consistency_summary",
            "relevance_summary",
            "coherence_summary",
        ],  # these are added by default if not specified
    )
    assert submit_resp.name == eval_name

    # wait for the async run to finish
    # try three times with linear backoff
    for i in range(1, 10):
        time.sleep(3 * i)

        # get the test run item
        test_run = mut.get_test_run(submit_resp.id)
        if test_run.status == "FINISHED":
            break
        assert test_run.status == "RUNNING"
    assert_metrics(test_run, num_rows=1)


def test_submit_batch_model_generation(
    rnd: str, okareo: Okareo, article_clf_scenario_set: ScenarioSetResponse
) -> None:
    def generation_rules(model_input: str) -> str:
        # simple generation rules to ensure consistent model outputs
        out = [s.split(" ")[0] for s in model_input.split(". ")]
        return " ".join(out)

    class GenerationModel(CustomModel):
        def invoke(self, input_value: Any) -> ModelInvocation:
            return ModelInvocation(
                model_prediction=generation_rules(input_value),
                model_input=input_value,
                model_output_metadata={"model_data": input_value},
                tool_calls=[{"function": "function"}],
            )

    mut = okareo.register_model(
        name=f"ci-custom-nlg-batch-{rnd}",
        model=GenerationModel(name="test_run_batch_model_generation - GenerationModel"),
        update=True,
    )
    run_async_resp = mut.submit_test(
        name=f"ci-custom-nlg-{rnd}",
        scenario=article_clf_scenario_set,
        test_run_type=TestRunType.NL_GENERATION,
        checks=["latency"],
    )

    # wait for the async run to finish
    # try three times with linear backoff
    for i in range(1, 10):
        time.sleep(3 * i)

        # get the test run item
        test_run = mut.get_test_run(run_async_resp.id)
        if test_run.status == "FINISHED":
            break
        assert test_run.status == "RUNNING"
    assert_metrics(test_run, num_rows=3, custom_dimensions=["latency"])



""" Contains shared errors types that can be raised from API functions """


class UnexpectedStatus(Exception):
    """Raised by api functions when the response status an undocumented status and Client.raise_on_unexpected_status is True"""

    def __init__(self, status_code: int, content: bytes):
        self.status_code = status_code
        self.content = content

        super().__init__(f"Unexpected status code: {status_code}")


__all__ = ["UnexpectedStatus"]



import random
from datetime import datetime
from typing import List, Union

import pytest
from okareo_tests.common import API_KEY, random_string

from okareo import Okareo
from okareo.model_under_test import CustomModel, ModelInvocation
from okareo_api_client.models import SeedData
from okareo_api_client.models.project_response import ProjectResponse
from okareo_api_client.models.scenario_set_create import ScenarioSetCreate
from okareo_api_client.models.test_run_item import TestRunItem
from okareo_api_client.models.test_run_type import TestRunType


@pytest.fixture(scope="module")
def rnd() -> str:
    return f'{random_string(5)} {datetime.now().strftime("%Y%m%d%H%M%S")}'


@pytest.fixture(scope="module")
def okareo_client() -> Okareo:
    return Okareo(api_key=API_KEY)


def test_get_projects(okareo_client: Okareo) -> None:
    projects = okareo_client.get_projects()
    assert projects
    assert isinstance(projects, List)
    assert len(projects) > 0
    assert projects[0].id
    assert projects[0].name


def test_create_project(okareo_client: Okareo) -> None:
    project = okareo_client.create_project(
        name="CI - Test P_Name", tags=["testT1", "testT2"]
    )
    assert project
    assert isinstance(project, ProjectResponse)
    assert project.id
    assert project.name == "CI - Test P_Name"
    assert project.tags == ["testT1", "testT2"]


def test_full_eval_cycle_in_new_project(rnd: str, okareo_client: Okareo) -> None:
    # use the same project to not overload the menu

    project = next(
        (
            item
            for item in okareo_client.get_projects()
            if item.name == "CI - test_full_eval_in_new_project"
        ),
        None,
    )

    if not project:
        project = okareo_client.create_project(
            name="CI - test_full_eval_in_new_project"
        )
        assert project.id

    scenario_set_create = ScenarioSetCreate(
        name=f"CI - test_full_eval_in_new_project {rnd}",
        seed_data=[
            SeedData(input_="sample input", result="returns"),
            SeedData(input_="sample input 2", result="pricing"),
        ],
        project_id=project.id,
    )
    response = okareo_client.create_scenario_set(scenario_set_create)

    assert response.project_id == project.id

    class ClassificationModel(CustomModel):
        def invoke(self, input_value: Union[dict, list, str]) -> ModelInvocation:
            prediction = random.choice(["returns", "complains", "pricing"])
            # return a tuple of (actual, overall model response context)
            return ModelInvocation(
                model_prediction=prediction,
                model_output_metadata={"labels": prediction, "confidence": 0.8},
            )

    mut = okareo_client.register_model(
        project_id=project.id,
        name=f"CI - from datetime import datetime {rnd}",
        model=ClassificationModel(name=f"CI - test_full_eval_in_new_project {rnd}"),
    )
    assert mut.mut_id
    assert mut.project_id == project.id

    # use the scenario id from one of the scenario set notebook examples
    assert isinstance(response.scenario_id, str)
    test_run_item = mut.run_test(
        scenario=response.scenario_id,
        name=f"CI test_full_eval_in_new_project {rnd}",
        test_run_type=TestRunType.MULTI_CLASS_CLASSIFICATION,
        calculate_metrics=True,
    )

    assert_valid_test_run(
        test_run_item, project.id
    )  # test_run inherits project_id from model

    test_run_get = mut.get_test_run(
        test_run_item.id
    )  # validate roudtrip to retrieve test run

    assert_valid_test_run(test_run_get, project.id)


def assert_valid_test_run(test_run_item: TestRunItem, project_id: str) -> None:
    assert test_run_item
    assert test_run_item.id
    assert test_run_item.model_metrics
    assert test_run_item.model_metrics.additional_properties.get("scores_by_label")
    assert test_run_item.project_id == project_id



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.test_data_point_item_metric_value import TestDataPointItemMetricValue


T = TypeVar("T", bound="TestDataPointItem")


@_attrs_define
class TestDataPointItem:
    """
    Attributes:
        id (str):
        scenario_data_point_id (str):
        test_run_id (str):
        metric_type (str):
        metric_value (TestDataPointItemMetricValue):
        tags (Union[Unset, List[str]]):
        checks (Union[Unset, Any]):
    """

    id: str
    scenario_data_point_id: str
    test_run_id: str
    metric_type: str
    metric_value: "TestDataPointItemMetricValue"
    tags: Union[Unset, List[str]] = UNSET
    checks: Union[Unset, Any] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        scenario_data_point_id = self.scenario_data_point_id
        test_run_id = self.test_run_id
        metric_type = self.metric_type
        metric_value = self.metric_value.to_dict()

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        checks = self.checks

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "scenario_data_point_id": scenario_data_point_id,
                "test_run_id": test_run_id,
                "metric_type": metric_type,
                "metric_value": metric_value,
            }
        )
        if tags is not UNSET:
            field_dict["tags"] = tags
        if checks is not UNSET:
            field_dict["checks"] = checks

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.test_data_point_item_metric_value import TestDataPointItemMetricValue

        d = src_dict.copy()
        id = d.pop("id")

        scenario_data_point_id = d.pop("scenario_data_point_id")

        test_run_id = d.pop("test_run_id")

        metric_type = d.pop("metric_type")

        metric_value = TestDataPointItemMetricValue.from_dict(d.pop("metric_value"))

        tags = cast(List[str], d.pop("tags", UNSET))

        checks = d.pop("checks", UNSET)

        test_data_point_item = cls(
            id=id,
            scenario_data_point_id=scenario_data_point_id,
            test_run_id=test_run_id,
            metric_type=metric_type,
            metric_value=metric_value,
            tags=tags,
            checks=checks,
        )

        test_data_point_item.additional_properties = d
        return test_data_point_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from okareo.checks import CodeBasedCheck


class Check(CodeBasedCheck):
    @staticmethod
    def evaluate(model_output: str, scenario_input: str, scenario_result: str, metadata: dict, model_input: str) -> bool:  # fmt: skip
        return (
            True
            if len(model_output + scenario_input + scenario_result + str(model_input))
            + len(metadata)
            >= 20
            else False
        )



import os
import uuid
from datetime import datetime

import pytest
from okareo_tests.common import API_KEY, OkareoAPIhost, integration
from pytest_httpx import HTTPXMock

from okareo import Okareo
from okareo_api_client.models.scenario_set_create import ScenarioSetCreate
from okareo_api_client.models.scenario_set_generate import ScenarioSetGenerate
from okareo_api_client.models.seed_data import SeedData
from okareo_api_client.types import UNSET


def test_can_instantiate() -> None:
    Okareo(API_KEY)


@pytest.fixture
def okareo_client(httpx_mock: HTTPXMock) -> Okareo:
    httpx_mock.add_response(
        json=[
            {
                "id": "0156f5d7-4ac4-4568-9d44-24750aa08d1a",
                "name": "Global",
                "onboarding_status": "onboarding_status",
                "tags": [],
                "additional_properties": {},
            }
        ],
        status_code=201,
    )
    return Okareo("foo", "http://mocked.com")


@integration
def test_create_scenario_set(httpx_mock: HTTPXMock, okareo_api: OkareoAPIhost) -> None:
    # Mocking a successful response
    mock_response = {
        "scenario_id": "scenario_id",
        "name": "test_scenario",
        "project_id": "test_id",
        "type": "test_type",
        "time_created": "2023-10-20T13:51:57.334956",
        # ... any other fields ...
    }

    if okareo_api.is_mock:
        httpx_mock.add_response(
            json=[
                {
                    "id": "0156f5d7-4ac4-4568-9d44-24750aa08d1a",
                    "name": "Global",
                    "onboarding_status": "onboarding_status",
                    "tags": [],
                    "additional_properties": {},
                }
            ],
            status_code=201,
        )
        httpx_mock.add_response(json=mock_response, status_code=201)

    okareo = Okareo(api_key=API_KEY, base_path=okareo_api.path)
    scenario_request = ScenarioSetCreate(
        name="test_scenario",
        seed_data=[SeedData(input_="foo", result="bar")],
        project_id="project_id" if okareo_api.is_mock else UNSET,
    )
    scenario_response = okareo.create_scenario_set(scenario_request)

    if okareo_api.is_mock:
        assert scenario_response.scenario_id == "scenario_id"
    else:
        assert scenario_response.scenario_id
        uuid.UUID(scenario_response.scenario_id)
    assert scenario_response.name == "test_scenario"


def test_register_model_raises_on_validation_error(
    okareo_client: Okareo, httpx_mock: HTTPXMock
) -> None:
    httpx_mock.add_response(json={"wrong": "response format"}, status_code=200)
    with pytest.raises(Exception, match="Unexpected status code"):
        okareo_client.register_model("this should fail", project_id="42")


def test_register_model_raises_on_empty_response(
    okareo_client: Okareo, httpx_mock: HTTPXMock
) -> None:
    httpx_mock.add_response(status_code=422)
    with pytest.raises(Exception, match="Expecting value"):
        okareo_client.register_model("this should fail", project_id="42")


def test_create_scenario_set_raises_on_validation_error(
    okareo_client: Okareo, httpx_mock: HTTPXMock
) -> None:
    httpx_mock.add_response(json={"wrong": "response format"}, status_code=200)

    seed_data = [
        SeedData(input_="are you able to set up in aws?", result="capabilities"),
    ]
    scenario_set_create = ScenarioSetCreate(
        name="my test scenario set x",
        seed_data=seed_data,
    )
    with pytest.raises(Exception, match="Unexpected"):
        okareo_client.create_scenario_set(create_request=scenario_set_create)


def test_upload_scenario_set(okareo_client: Okareo, httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(
        json={
            "scenario_id": "created-id",
            "project_id": "new-project-id",
            "time_created": datetime.now().isoformat(),
            "type": "seed",
        },
        status_code=200,
    )
    okareo_client.upload_scenario_set(
        scenario_name="my-test-scenario-1",
        file_path=os.path.join(os.path.dirname(__file__), "webbizz_class_seed.jsonl"),
    )


def test_generate_scenario_set(okareo_client: Okareo, httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(
        json={
            "scenario_id": "created-id",
            "project_id": "new-project-id",
            "time_created": datetime.now().isoformat(),
            "type": "seed",
        },
        status_code=201,
    )

    okareo_client.generate_scenario_set(
        ScenarioSetGenerate(
            source_scenario_id="just-a-random-scenario-id",
            name="my generation foo",
            number_examples=22,
        )
    )


def test_get_scenario_data_points(okareo_client: Okareo, httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(json={}, status_code=200)
    okareo_client.get_scenario_data_points(scenario_id="scenario-id")


def test_get_all_checks(okareo_client: Okareo, httpx_mock: HTTPXMock) -> None:
    httpx_mock.add_response(json={}, status_code=201)
    okareo_client.get_all_checks()



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.comparison_operator import ComparisonOperator
from ..models.datapoint_field import DatapointField
from ..types import UNSET, Unset

T = TypeVar("T", bound="FilterCondition")


@_attrs_define
class FilterCondition:
    """
    Attributes:
        field (DatapointField): An enumeration.
        operator (ComparisonOperator): An enumeration.
        value (str): Value to compare against
        field_key (Union[Unset, str]): Key of the User Metadata field to filter on. Ignored for other 'field' values
    """

    field: DatapointField
    operator: ComparisonOperator
    value: str
    field_key: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field = self.field.value

        operator = self.operator.value

        value = self.value
        field_key = self.field_key

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "field": field,
                "operator": operator,
                "value": value,
            }
        )
        if field_key is not UNSET:
            field_dict["field_key"] = field_key

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        field = DatapointField(d.pop("field"))

        operator = ComparisonOperator(d.pop("operator"))

        value = d.pop("value")

        field_key = d.pop("field_key", UNSET)

        filter_condition = cls(
            field=field,
            operator=operator,
            value=value,
            field_key=field_key,
        )

        filter_condition.additional_properties = d
        return filter_condition

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import time
from datetime import datetime
from unittest import mock

import litellm  # type: ignore
from litellm import completion
from okareo_tests.common import API_KEY, OkareoAPIhost, integration, random_string
from pytest_httpx import HTTPXMock

from okareo import Okareo
from okareo.litellm_logger import LiteLLMLogger, LiteLLMProxyLogger
from okareo_api_client.models.datapoint_search import DatapointSearch
from okareo_api_client.models.model_under_test_response import ModelUnderTestResponse


def get_mut_response() -> dict:
    return ModelUnderTestResponse(
        "id",
        "my-project",
        "langchain_test",
        ["ci-run"],
        datetime.now().isoformat(),
    ).to_dict()


@integration
def dtest_litellm_baselogger(httpx_mock: HTTPXMock, okareo_api: OkareoAPIhost) -> None:
    context_token = random_string(10)
    if okareo_api.is_mock:
        httpx_mock.add_response(json=get_mut_response(), status_code=201)

    with mock.patch.object(Okareo.__init__, "__defaults__", (okareo_api.path, 100)):
        handler = LiteLLMLogger(
            api_key=API_KEY,
            mut_name="test_litellm_baselogger-" + context_token,
            context_token=context_token,
        )

    litellm.callbacks = [handler]  # type: ignore
    model = "gpt-3.5-turbo"
    response = "ModelResponse(id='chatcmpl-90E2ohwLpPXSIuPRzm0fc905yVjZh', choices=[Choices(finish_reason='stop', index=0, message=Message(content='Barack Obama is an American politician who served as the 44th President of the United States from 2009 to 2017. He is a member of the Democratic Party and was the first African American to be elected to the presidency. Before becoming president, he served as a U.S. Senator from Illinois and worked as a community organizer and civil rights attorney. He is widely regarded as one of the most influential and charismatic leaders in modern American history.', role='assistant'))], created=1709841014, model='gpt-3.5-turbo-0125', object='chat.completion', system_fingerprint='fp_2b778c6b35', usage=Usage(completion_tokens=92, prompt_tokens=12, total_tokens=104))"
    messages = [{"content": "who is obama?", "role": "user"}]
    response = completion(model=model, messages=messages, mock_response=response)

    # needs to wait for callback to finish
    time.sleep(2)

    if okareo_api.is_mock:
        requests = httpx_mock.get_requests()
        assert requests[0].method == "POST"
        assert "/v0/register_model" in requests[0].url.path
        assert requests[1].method == "POST"
        assert "/v0/datapoints" in requests[1].url.path
    else:
        okareo = Okareo(api_key=API_KEY, base_path=okareo_api.path)
        dp = okareo.find_datapoints(DatapointSearch(context_token=context_token))
        assert isinstance(dp, list)


@integration
def dtest_litellm_openailogger(
    httpx_mock: HTTPXMock, okareo_api: OkareoAPIhost
) -> None:
    context_token = random_string(10)
    if okareo_api.is_mock:
        httpx_mock.add_response(json=get_mut_response(), status_code=201)

    with mock.patch.object(Okareo.__init__, "__defaults__", (okareo_api.path, 100)):
        handler = LiteLLMProxyLogger(
            api_key=API_KEY,
            mut_name="test_litellm_openailogger-" + context_token,
            context_token=context_token,
        )

    litellm.callbacks = [handler]  # type: ignore
    model = "gpt-3.5-turbo"
    response = "ModelResponse(id='chatcmpl-90E2ohwLpPXSIuPRzm0fc905yVjZh', choices=[Choices(finish_reason='stop', index=0, message=Message(content='Barack Obama is an American politician who served as the 44th President of the United States from 2009 to 2017. He is a member of the Democratic Party and was the first African American to be elected to the presidency. Before becoming president, he served as a U.S. Senator from Illinois and worked as a community organizer and civil rights attorney. He is widely regarded as one of the most influential and charismatic leaders in modern American history.', role='assistant'))], created=1709841014, model='gpt-3.5-turbo-0125', object='chat.completion', system_fingerprint='fp_2b778c6b35', usage=Usage(completion_tokens=92, prompt_tokens=12, total_tokens=104))"
    messages = [{"content": "who is obama?", "role": "user"}]
    response = completion(model=model, messages=messages, mock_response=response)

    # needs to wait for callback to finish
    time.sleep(2)

    if okareo_api.is_mock:
        requests = httpx_mock.get_requests()
        assert requests[0].method == "POST"
        assert "/v0/register_model" in requests[0].url.path
        assert requests[1].method == "POST"
        assert "/v0/datapoints" in requests[1].url.path
    else:
        okareo = Okareo(api_key=API_KEY, base_path=okareo_api.path)
        dp = okareo.find_datapoints(DatapointSearch(context_token=context_token))
        assert isinstance(dp, list)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.evaluator_brief_response_check_config import EvaluatorBriefResponseCheckConfig


T = TypeVar("T", bound="EvaluatorBriefResponse")


@_attrs_define
class EvaluatorBriefResponse:
    """
    Attributes:
        id (Union[Unset, str]):
        name (Union[Unset, str]):
        description (Union[Unset, str]):  Default: ''.
        output_data_type (Union[Unset, str]):  Default: ''.
        time_created (Union[Unset, datetime.datetime]):
        check_config (Union[Unset, EvaluatorBriefResponseCheckConfig]):
        is_predefined (Union[Unset, bool]):
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = ""
    output_data_type: Union[Unset, str] = ""
    time_created: Union[Unset, datetime.datetime] = UNSET
    check_config: Union[Unset, "EvaluatorBriefResponseCheckConfig"] = UNSET
    is_predefined: Union[Unset, bool] = False
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        description = self.description
        output_data_type = self.output_data_type
        time_created: Union[Unset, str] = UNSET
        if not isinstance(self.time_created, Unset):
            time_created = self.time_created.isoformat()

        check_config: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.check_config, Unset):
            check_config = self.check_config.to_dict()

        is_predefined = self.is_predefined

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if output_data_type is not UNSET:
            field_dict["output_data_type"] = output_data_type
        if time_created is not UNSET:
            field_dict["time_created"] = time_created
        if check_config is not UNSET:
            field_dict["check_config"] = check_config
        if is_predefined is not UNSET:
            field_dict["is_predefined"] = is_predefined

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.evaluator_brief_response_check_config import EvaluatorBriefResponseCheckConfig

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        output_data_type = d.pop("output_data_type", UNSET)

        _time_created = d.pop("time_created", UNSET)
        time_created: Union[Unset, datetime.datetime]
        if isinstance(_time_created, Unset):
            time_created = UNSET
        else:
            time_created = isoparse(_time_created)

        _check_config = d.pop("check_config", UNSET)
        check_config: Union[Unset, EvaluatorBriefResponseCheckConfig]
        if isinstance(_check_config, Unset):
            check_config = UNSET
        else:
            check_config = EvaluatorBriefResponseCheckConfig.from_dict(_check_config)

        is_predefined = d.pop("is_predefined", UNSET)

        evaluator_brief_response = cls(
            id=id,
            name=name,
            description=description,
            output_data_type=output_data_type,
            time_created=time_created,
            check_config=check_config,
            is_predefined=is_predefined,
        )

        evaluator_brief_response.additional_properties = d
        return evaluator_brief_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import inspect
from abc import ABC, abstractmethod
from enum import Enum
from typing import Optional, Union

from attrs import define as _attrs_define


@_attrs_define
class CheckResponse:
    """Response object for code-based checks.

    Attributes:
        score: The numeric or boolean result of the check.
        explanation: Optional string explanation for the score, if applicable.
        metadata: Optional dictionary containing metadata about the check. E.g., output tokens, input tokens, latency, cost, etc.
    """

    score: Union[bool, int, float]
    explanation: Optional[str] = None
    metadata: Optional[dict] = None


class BaseCheck(ABC):
    """
    Base class for defining checks
    """

    @staticmethod
    @abstractmethod
    def evaluate(
        model_output: str,
        scenario_input: str,
        scenario_result: str,
        metadata: dict,
        model_input: str,
    ) -> Union[bool, int, float, CheckResponse, tuple[Union[bool, int, float], str]]:
        """
        Evaluate your model output, scenario input, scenario result, metadata, and model_input to determine if the data should pass or fail the check.
        """

    def check_config(self) -> dict:
        """
        Returns a dictionary of configuration parameters that will be passed to the API.
        """
        return {}


class CheckOutputType(Enum):
    """
    Enum for the type of output that the check will produce. This is used in ModelBasedCheck.
    """

    SCORE = "score"
    PASS_FAIL = "pass_fail"


class ModelBasedCheck(BaseCheck):
    """
    Check that uses a prompt template to evaluate the data.

    The prompt template should be a string that includes at least one of
    the following placeholders, which will be replaced with the actual values:
    - `{model_input}` -> corresponds to the model's input
    - `{generation}` -> corresponds to the model's output
    - `{scenario_input}` -> corresponds to the scenario input
    - `{scenario_result}` -> corresponds to the scenario result

    Example of how a template could be used: "Count the words in the following: `{generation}`"

    The check output type should be one of the following:
    - CheckOutputType.SCORE -> this template should ask prompt the model a score (single number)
    - CheckOutputType.PASS_FAIL -> this template should prompt the model for a boolean value (True/False)
    """

    def __init__(self, prompt_template: str, check_type: CheckOutputType):
        """Initialize the check with a prompt template and check type"""
        self.prompt_template = prompt_template
        self.check_type = check_type.value

    def check_config(self) -> dict:
        return {"prompt_template": self.prompt_template, "type": self.check_type}

    @staticmethod
    def evaluate(
        model_output: str,
        scenario_input: str,
        scenario_result: str,
        metadata: dict,
        model_input: str,
    ) -> CheckResponse:
        raise NotImplementedError("Evaluate method is handled on server.")


class CodeBasedCheck(BaseCheck):
    """
    A check that uses code to evaluate the data

    To use this check:
    1. Create a new Python file (not in a notebook).
    2. In this file, define a class named 'Check' that inherits from CodeBasedCheck.
    3. Implement the `evaluate` method in your Check class.
    4. Include any additional code used by your check in the same file.

    Example:
    ```python
    # In my_custom_check.py
    from okareo.checks import CodeBasedCheck, CheckResponse

    class Check(CodeBasedCheck):
        @staticmethod
        @abstractmethod
        def evaluate(
            model_output: str, scenario_input: str, scenario_result: str, metadata: dict, model_input: str
        ) -> CheckResponse:
            # Your code here
            return CheckResponse(score=True, explanation="This is an explanation of my score.")
    ```

    The `evaluate` method parameters can be any subset of the possible parameters.
    For example, if your check only needs `model_output` and `model_input`, then you can write a method like this:
    ```python
    def evaluate(
        model_output: str, model_input: str
    ) -> CheckResponse:
        # ...your check logic here...
    ```
    """

    def check_config(self) -> dict:
        module = inspect.getmodule(self)
        if module is None:
            raise ValueError("Unable to find module for check class")
        try:
            source = inspect.getsource(module)
        except OSError as e:
            raise ValueError(
                "Unable to read source code for check class. Please place the check class in a separate file."
            ) from e
        output_type = inspect.signature(self.evaluate).return_annotation.__name__
        return {"code_contents": source, "type": output_type}



import os
import secrets
import time
from typing import Any

import pytest
import requests  # type: ignore
from google.protobuf.json_format import ParseDict  # type: ignore
from opentelemetry.proto.collector.trace.v1.trace_service_pb2 import (
    ExportTraceServiceRequest,  # type: ignore
)


@pytest.fixture
def api_key():  # type: ignore
    """Get API key from environment variable."""
    api_key = os.environ.get("OKAREO_API_KEY")
    if not api_key:
        pytest.skip("OKAREO_API_KEY environment variable not set")
    return api_key


@pytest.fixture
def base_url():  # type: ignore
    """Get base URL from environment variable or use default."""
    return os.environ.get("BASE_URL", "https://api.okareo.com")


def generate_random_string():  # type: ignore
    """Generate a unique random string for testing."""
    return f"trace_test_{secrets.token_hex(8)}"


def verify_span_recorded(random_string: Any, api_key: Any, base_url: Any) -> Any:
    """Verify the span with the random string was recorded in Okareo."""
    # Wait for data to propagate
    time.sleep(15)

    headers = {"Content-Type": "application/json", "api-key": api_key}

    # Get project ID
    project_response = requests.get(f"{base_url}/v0/projects", headers=headers)

    project_id = project_response.json()[0]["id"]

    # Try a few times with backoff
    max_attempts = 3
    for attempt in range(max_attempts):
        try:
            # Query for the span containing our random string
            payload = {
                "limit": 5,
                "filters": [
                    {"field": "input", "operator": "contains", "value": random_string}
                ],
                "project_id": project_id,
            }

            response = requests.post(
                f"{base_url}/v0/find_spans_filter", headers=headers, json=payload
            )

            data = response.json()
            if len(data) > 0:
                print(f"Found span with random string: {random_string}")
                return True

            # If not found and not last attempt, wait longer
            if attempt < max_attempts - 1:
                print(f"Span with {random_string} not found, retrying...")
                time.sleep(10 * (attempt + 1))

        except Exception as e:
            print(f"Error verifying span: {e}")
            if attempt < max_attempts - 1:
                time.sleep(5 * (attempt + 1))

    print(f"Failed to find span with random string: {random_string}")
    return False


def test_traces_endpoint_with_verification(api_key: Any, base_url: Any) -> Any:
    """Test sending trace data to the /traces endpoint with verification."""

    # Generate random identifier for this test
    random_string = generate_random_string()
    print(f"Using random string: {random_string}")

    # Prepare sample trace data with the random string
    sample_data = {
        "resourceSpans": [
            {
                "resource": {
                    "attributes": [
                        {"key": "service.name", "value": {"stringValue": "litellm"}},
                        {
                            "key": "deployment.environment",
                            "value": {"stringValue": "production"},
                        },
                        {"key": "model_id", "value": {"stringValue": "litellm"}},
                    ]
                },
                "scopeSpans": [
                    {
                        "scope": {"name": "litellm"},
                        "spans": [
                            {
                                "traceId": "+Ma464bO5ON9+YlhFsG9Gg==",
                                "spanId": "y1Ygv5dY0jM=",
                                "parentSpanId": "DJfB8MovyNg=",
                                "name": f"litellm_request_{random_string}",  # Include random string in name
                                "kind": 1,
                                "startTimeUnixNano": "1746493671703133952",
                                "endTimeUnixNano": "1746493678456763904",
                                "attributes": [
                                    {
                                        "key": "gen_ai.request.model",
                                        "value": {
                                            "stringValue": "gemini-2.5-pro-exp-03-25"
                                        },
                                    },
                                    {
                                        "key": "llm.request.type",
                                        "value": {"stringValue": "acompletion"},
                                    },
                                    {
                                        "key": "llm.messages",
                                        "value": {
                                            "stringValue": f'[{{"content": "Test message with {random_string}", "role": "user"}}]'
                                        },
                                    },
                                    {
                                        "key": "test_identifier",  # Add dedicated attribute for the random string
                                        "value": {"stringValue": random_string},
                                    },
                                    {
                                        "key": "gen_ai.system",
                                        "value": {"stringValue": "vertex_ai"},
                                    },
                                    {
                                        "key": "gen_ai.response.id",
                                        "value": {
                                            "stringValue": f"chatcmpl-{random_string}"
                                        },
                                    },
                                    {
                                        "key": "llm.usage.total_tokens",
                                        "value": {"intValue": "574"},
                                    },
                                    {
                                        "key": "gen_ai.usage.completion_tokens",
                                        "value": {"intValue": "10"},
                                    },
                                    {
                                        "key": "gen_ai.usage.prompt_tokens",
                                        "value": {"intValue": "1"},
                                    },
                                    {
                                        "key": "SpanAttributes.LLM_PROMPTS.0.content",
                                        "value": {
                                            "stringValue": f"Test with {random_string}"
                                        },
                                    },
                                    {
                                        "key": "SpanAttributes.LLM_COMPLETIONS.0.content",
                                        "value": {
                                            "stringValue": f"Response to {random_string}"
                                        },
                                    },
                                ],
                                "status": {"code": 1},
                                "flags": 256,
                            }
                        ],
                    }
                ],
            }
        ]
    }

    # Convert JSON to protobuf
    trace_request = ExportTraceServiceRequest()
    ParseDict(sample_data, trace_request)

    # Serialize to binary format
    serialized_data = trace_request.SerializeToString()

    # Set up headers
    headers = {"Content-Type": "application/x-protobuf", "api-key": api_key}

    # Send request
    response = requests.post(
        f"{base_url}/v0/traces", headers=headers, data=serialized_data
    )

    # Assert response
    assert (
        response.status_code == 201
    ), f"Expected status code 201, got {response.status_code}: {response.text}"

    # Verify the response data
    try:
        response_data = response.json()
        assert response_data.get("status") == "success"
        assert "message" in response_data
        print(f"Response: {response_data}")
    except ValueError:
        # If response is not JSON, just check status code
        pass

    # Verify the span was recorded in Okareo
    assert verify_span_recorded(
        random_string, api_key, base_url
    ), f"Span with random string '{random_string}' was not found in Okareo"



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.scenario_data_poin_response import ScenarioDataPoinResponse
    from ..models.seed_data import SeedData


T = TypeVar("T", bound="ScenarioSetResponse")


@_attrs_define
class ScenarioSetResponse:
    """
    Attributes:
        project_id (str):
        time_created (datetime.datetime):
        type (str):
        scenario_id (Union[Unset, str]):
        tags (Union[Unset, List[str]]):
        name (Union[Unset, str]):
        seed_data (Union[Unset, List['SeedData']]):
        scenario_data (Union[Unset, List['ScenarioDataPoinResponse']]):
        failed_data (Union[Unset, List['ScenarioDataPoinResponse']]):
        scenario_count (Union[Unset, int]):
        scenario_input (Union[Unset, List[str]]):
        app_link (Union[Unset, str]): This URL links to the Okareo webpage for this scenario set Default: ''.
        warning (Union[Unset, str]):
    """

    project_id: str
    time_created: datetime.datetime
    type: str
    scenario_id: Union[Unset, str] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    name: Union[Unset, str] = UNSET
    seed_data: Union[Unset, List["SeedData"]] = UNSET
    scenario_data: Union[Unset, List["ScenarioDataPoinResponse"]] = UNSET
    failed_data: Union[Unset, List["ScenarioDataPoinResponse"]] = UNSET
    scenario_count: Union[Unset, int] = 0
    scenario_input: Union[Unset, List[str]] = UNSET
    app_link: Union[Unset, str] = ""
    warning: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        project_id = self.project_id
        time_created = self.time_created.isoformat()

        type = self.type
        scenario_id = self.scenario_id
        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        name = self.name
        seed_data: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.seed_data, Unset):
            seed_data = []
            for seed_data_item_data in self.seed_data:
                seed_data_item = seed_data_item_data.to_dict()

                seed_data.append(seed_data_item)

        scenario_data: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.scenario_data, Unset):
            scenario_data = []
            for scenario_data_item_data in self.scenario_data:
                scenario_data_item = scenario_data_item_data.to_dict()

                scenario_data.append(scenario_data_item)

        failed_data: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.failed_data, Unset):
            failed_data = []
            for failed_data_item_data in self.failed_data:
                failed_data_item = failed_data_item_data.to_dict()

                failed_data.append(failed_data_item)

        scenario_count = self.scenario_count
        scenario_input: Union[Unset, List[str]] = UNSET
        if not isinstance(self.scenario_input, Unset):
            scenario_input = self.scenario_input

        app_link = self.app_link
        warning = self.warning

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project_id": project_id,
                "time_created": time_created,
                "type": type,
            }
        )
        if scenario_id is not UNSET:
            field_dict["scenario_id"] = scenario_id
        if tags is not UNSET:
            field_dict["tags"] = tags
        if name is not UNSET:
            field_dict["name"] = name
        if seed_data is not UNSET:
            field_dict["seed_data"] = seed_data
        if scenario_data is not UNSET:
            field_dict["scenario_data"] = scenario_data
        if failed_data is not UNSET:
            field_dict["failed_data"] = failed_data
        if scenario_count is not UNSET:
            field_dict["scenario_count"] = scenario_count
        if scenario_input is not UNSET:
            field_dict["scenario_input"] = scenario_input
        if app_link is not UNSET:
            field_dict["app_link"] = app_link
        if warning is not UNSET:
            field_dict["warning"] = warning

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.scenario_data_poin_response import ScenarioDataPoinResponse
        from ..models.seed_data import SeedData

        d = src_dict.copy()
        project_id = d.pop("project_id")

        time_created = isoparse(d.pop("time_created"))

        type = d.pop("type")

        scenario_id = d.pop("scenario_id", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        name = d.pop("name", UNSET)

        seed_data = []
        _seed_data = d.pop("seed_data", UNSET)
        for seed_data_item_data in _seed_data or []:
            seed_data_item = SeedData.from_dict(seed_data_item_data)

            seed_data.append(seed_data_item)

        scenario_data = []
        _scenario_data = d.pop("scenario_data", UNSET)
        for scenario_data_item_data in _scenario_data or []:
            scenario_data_item = ScenarioDataPoinResponse.from_dict(scenario_data_item_data)

            scenario_data.append(scenario_data_item)

        failed_data = []
        _failed_data = d.pop("failed_data", UNSET)
        for failed_data_item_data in _failed_data or []:
            failed_data_item = ScenarioDataPoinResponse.from_dict(failed_data_item_data)

            failed_data.append(failed_data_item)

        scenario_count = d.pop("scenario_count", UNSET)

        scenario_input = cast(List[str], d.pop("scenario_input", UNSET))

        app_link = d.pop("app_link", UNSET)

        warning = d.pop("warning", UNSET)

        scenario_set_response = cls(
            project_id=project_id,
            time_created=time_created,
            type=type,
            scenario_id=scenario_id,
            tags=tags,
            name=name,
            seed_data=seed_data,
            scenario_data=scenario_data,
            failed_data=failed_data,
            scenario_count=scenario_count,
            scenario_input=scenario_input,
            app_link=app_link,
            warning=warning,
        )

        scenario_set_response.additional_properties = d
        return scenario_set_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any

from autogen import ConversableAgent  # type: ignore
from okareo_tests.common import API_KEY, OPENAI_API_KEY, random_string

from okareo import Okareo
from okareo.autogen_logger import AutogenLogger
from okareo_api_client.models.datapoint_search import DatapointSearch


def get_logger_config() -> dict[str, Any]:
    logger_config = {
        "api_key": API_KEY,
        "tags": ["autogen-test"],
        "group_name": f"test_autogen_group_{random_string(5)}",
    }
    return logger_config


def get_gpt_config() -> dict[str, Any]:
    gpt4_config_list = [
        {
            "model": "gpt-4-0125-preview",
            "api_key": OPENAI_API_KEY,
        }
    ]

    gpt4_config = {
        "cache_seed": 42,  # change the cache_seed for different trials
        "temperature": 0,
        "config_list": gpt4_config_list,
        "timeout": 120,
    }

    return gpt4_config


def call_autogen_agents(message: str) -> None:
    gpt_config = get_gpt_config()

    cathy = ConversableAgent(
        "cathy",
        system_message="Your name is Cathy and you are a part of a duo of comedians.",
        llm_config=gpt_config,
        human_input_mode="NEVER",  # Never ask for human input.
    )

    joe = ConversableAgent(
        "joe",
        system_message="Your name is Joe and you are a part of a duo of comedians.",
        llm_config=gpt_config,
        human_input_mode="NEVER",  # Never ask for human input.
    )

    joe.initiate_chat(cathy, message=message, max_turns=1)


def test_autogen_logger() -> None:
    logger_config = get_logger_config()
    autogen_logger = AutogenLogger(logger_config)

    with autogen_logger:
        call_autogen_agents("Tell me a joke")

    session_id_0 = autogen_logger.logger.session_id
    okareo = Okareo(api_key=API_KEY)
    dp = okareo.find_datapoints(DatapointSearch(context_token=session_id_0))

    assert isinstance(dp, list)



from typing import List, Union

from okareo_api_client.models.test_run_item import TestRunItem
from okareo_api_client.types import Unset


def assert_scores_geval(scores: dict) -> None:
    dimension_keys = [
        "consistency_summary",
        "coherence_summary",
        "fluency_summary",
        "relevance_summary",
    ]
    for dimension in dimension_keys:
        assert dimension in scores
        assert isinstance(scores[dimension], float)
        # Due to the imprecision of logprob calculation and python floats, the value can sometimes be 5.0000000000001
        assert 1 <= scores[dimension] <= 5.001


def assert_scores(scores: dict, custom_dimensions: List[str]) -> None:
    # Assert non-explanation scores specified in 'custom_dimensions' are present in 'scores'
    dimension_keys = custom_dimensions
    skip_keys = ["scenario_index", "test_id"]
    assert len(dimension_keys) == len(
        [k for k in scores.keys() if k not in skip_keys and "__explanation" not in k]
    )
    for dimension in dimension_keys:
        assert dimension in scores


def assert_metrics(
    run_resp: TestRunItem,
    custom_dimensions: Union[List[str], None] = None,
    num_rows: int = 3,
) -> None:
    assert run_resp.model_metrics is not None and not isinstance(
        run_resp.model_metrics, Unset
    )
    metrics_dict = run_resp.model_metrics.to_dict()

    assert metrics_dict["mean_scores"] is not None
    if custom_dimensions is not None:
        assert_scores(metrics_dict["mean_scores"], custom_dimensions)
    else:
        assert_scores_geval(metrics_dict["mean_scores"])
    assert metrics_dict["scores_by_row"] is not None
    assert len(metrics_dict["scores_by_row"]) == num_rows
    for row in metrics_dict["scores_by_row"]:
        if custom_dimensions is not None:
            assert_scores(row, custom_dimensions)
        else:
            assert_scores_geval(row)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.test_custom_endpoint_response_end_session_raw_response import (
        TestCustomEndpointResponseEndSessionRawResponse,
    )
    from ..models.test_custom_endpoint_response_next_message_raw_response import (
        TestCustomEndpointResponseNextMessageRawResponse,
    )
    from ..models.test_custom_endpoint_response_start_session_raw_response import (
        TestCustomEndpointResponseStartSessionRawResponse,
    )


T = TypeVar("T", bound="TestCustomEndpointResponse")


@_attrs_define
class TestCustomEndpointResponse:
    """
    Attributes:
        start_session_raw_response (Union[Unset, TestCustomEndpointResponseStartSessionRawResponse]): Raw response from
            the start session endpoint, if applicable.
        next_message_raw_response (Union[Unset, TestCustomEndpointResponseNextMessageRawResponse]): Raw response from
            the next message endpoint.
        end_session_raw_response (Union[Unset, TestCustomEndpointResponseEndSessionRawResponse]): Raw response from the
            end session endpoint, if applicable.
    """

    start_session_raw_response: Union[Unset, "TestCustomEndpointResponseStartSessionRawResponse"] = UNSET
    next_message_raw_response: Union[Unset, "TestCustomEndpointResponseNextMessageRawResponse"] = UNSET
    end_session_raw_response: Union[Unset, "TestCustomEndpointResponseEndSessionRawResponse"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        start_session_raw_response: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.start_session_raw_response, Unset):
            start_session_raw_response = self.start_session_raw_response.to_dict()

        next_message_raw_response: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.next_message_raw_response, Unset):
            next_message_raw_response = self.next_message_raw_response.to_dict()

        end_session_raw_response: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.end_session_raw_response, Unset):
            end_session_raw_response = self.end_session_raw_response.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if start_session_raw_response is not UNSET:
            field_dict["start_session_raw_response"] = start_session_raw_response
        if next_message_raw_response is not UNSET:
            field_dict["next_message_raw_response"] = next_message_raw_response
        if end_session_raw_response is not UNSET:
            field_dict["end_session_raw_response"] = end_session_raw_response

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.test_custom_endpoint_response_end_session_raw_response import (
            TestCustomEndpointResponseEndSessionRawResponse,
        )
        from ..models.test_custom_endpoint_response_next_message_raw_response import (
            TestCustomEndpointResponseNextMessageRawResponse,
        )
        from ..models.test_custom_endpoint_response_start_session_raw_response import (
            TestCustomEndpointResponseStartSessionRawResponse,
        )

        d = src_dict.copy()
        _start_session_raw_response = d.pop("start_session_raw_response", UNSET)
        start_session_raw_response: Union[Unset, TestCustomEndpointResponseStartSessionRawResponse]
        if isinstance(_start_session_raw_response, Unset):
            start_session_raw_response = UNSET
        else:
            start_session_raw_response = TestCustomEndpointResponseStartSessionRawResponse.from_dict(
                _start_session_raw_response
            )

        _next_message_raw_response = d.pop("next_message_raw_response", UNSET)
        next_message_raw_response: Union[Unset, TestCustomEndpointResponseNextMessageRawResponse]
        if isinstance(_next_message_raw_response, Unset):
            next_message_raw_response = UNSET
        else:
            next_message_raw_response = TestCustomEndpointResponseNextMessageRawResponse.from_dict(
                _next_message_raw_response
            )

        _end_session_raw_response = d.pop("end_session_raw_response", UNSET)
        end_session_raw_response: Union[Unset, TestCustomEndpointResponseEndSessionRawResponse]
        if isinstance(_end_session_raw_response, Unset):
            end_session_raw_response = UNSET
        else:
            end_session_raw_response = TestCustomEndpointResponseEndSessionRawResponse.from_dict(
                _end_session_raw_response
            )

        test_custom_endpoint_response = cls(
            start_session_raw_response=start_session_raw_response,
            next_message_raw_response=next_message_raw_response,
            end_session_raw_response=end_session_raw_response,
        )

        test_custom_endpoint_response.additional_properties = d
        return test_custom_endpoint_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.test_run_type import TestRunType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.evaluation_payload_metrics_kwargs import EvaluationPayloadMetricsKwargs


T = TypeVar("T", bound="EvaluationPayload")


@_attrs_define
class EvaluationPayload:
    """
    Attributes:
        scenario_id (Union[Unset, str]): ID of the scenario set to evaluate
        datapoint_ids (Union[Unset, List[str]]): List of datapoint IDs to filter by
        metrics_kwargs (Union[Unset, EvaluationPayloadMetricsKwargs]): Dictionary of metrics to be measured
        filter_group_id (Union[Unset, str]): ID of the datapoint filter group to apply
        tags (Union[Unset, List[str]]): Tags are strings that can be used to filter test runs in the Okareo app
        checks (Union[Unset, List[str]]): List of checks to include in the test run.
        type (Union[Unset, TestRunType]): An enumeration. Default: TestRunType.MULTI_CLASS_CLASSIFICATION.
        name (Union[Unset, str]): Name of the test run
        project_id (Union[Unset, str]): Project ID
    """

    scenario_id: Union[Unset, str] = UNSET
    datapoint_ids: Union[Unset, List[str]] = UNSET
    metrics_kwargs: Union[Unset, "EvaluationPayloadMetricsKwargs"] = UNSET
    filter_group_id: Union[Unset, str] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    checks: Union[Unset, List[str]] = UNSET
    type: Union[Unset, TestRunType] = TestRunType.MULTI_CLASS_CLASSIFICATION
    name: Union[Unset, str] = UNSET
    project_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        scenario_id = self.scenario_id
        datapoint_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.datapoint_ids, Unset):
            datapoint_ids = self.datapoint_ids

        metrics_kwargs: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metrics_kwargs, Unset):
            metrics_kwargs = self.metrics_kwargs.to_dict()

        filter_group_id = self.filter_group_id
        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        checks: Union[Unset, List[str]] = UNSET
        if not isinstance(self.checks, Unset):
            checks = self.checks

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        name = self.name
        project_id = self.project_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if scenario_id is not UNSET:
            field_dict["scenario_id"] = scenario_id
        if datapoint_ids is not UNSET:
            field_dict["datapoint_ids"] = datapoint_ids
        if metrics_kwargs is not UNSET:
            field_dict["metrics_kwargs"] = metrics_kwargs
        if filter_group_id is not UNSET:
            field_dict["filter_group_id"] = filter_group_id
        if tags is not UNSET:
            field_dict["tags"] = tags
        if checks is not UNSET:
            field_dict["checks"] = checks
        if type is not UNSET:
            field_dict["type"] = type
        if name is not UNSET:
            field_dict["name"] = name
        if project_id is not UNSET:
            field_dict["project_id"] = project_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.evaluation_payload_metrics_kwargs import EvaluationPayloadMetricsKwargs

        d = src_dict.copy()
        scenario_id = d.pop("scenario_id", UNSET)

        datapoint_ids = cast(List[str], d.pop("datapoint_ids", UNSET))

        _metrics_kwargs = d.pop("metrics_kwargs", UNSET)
        metrics_kwargs: Union[Unset, EvaluationPayloadMetricsKwargs]
        if isinstance(_metrics_kwargs, Unset):
            metrics_kwargs = UNSET
        else:
            metrics_kwargs = EvaluationPayloadMetricsKwargs.from_dict(_metrics_kwargs)

        filter_group_id = d.pop("filter_group_id", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        checks = cast(List[str], d.pop("checks", UNSET))

        _type = d.pop("type", UNSET)
        type: Union[Unset, TestRunType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = TestRunType(_type)

        name = d.pop("name", UNSET)

        project_id = d.pop("project_id", UNSET)

        evaluation_payload = cls(
            scenario_id=scenario_id,
            datapoint_ids=datapoint_ids,
            metrics_kwargs=metrics_kwargs,
            filter_group_id=filter_group_id,
            tags=tags,
            checks=checks,
            type=type,
            name=name,
            project_id=project_id,
        )

        evaluation_payload.additional_properties = d
        return evaluation_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.filter_condition import FilterCondition


T = TypeVar("T", bound="DatapointFilterUpdate")


@_attrs_define
class DatapointFilterUpdate:
    """
    Attributes:
        filters (Union[Unset, List['FilterCondition']]): List of filter conditions to apply
        name (Union[Unset, str]): Optional name describing this filter
        description (Union[Unset, str]): Optional description of the filter
        checks (Union[Unset, List[str]]): Optional list of checks to apply to datapoints in the filter
        slack_enabled (Union[Unset, bool]): Whether to enable Slack notifications for this filter group.
        email_enabled (Union[Unset, bool]): Whether to enable Email notifications for this filter group.
        project_id (Union[Unset, str]): Project ID these filters belong to
    """

    filters: Union[Unset, List["FilterCondition"]] = UNSET
    name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    checks: Union[Unset, List[str]] = UNSET
    slack_enabled: Union[Unset, bool] = UNSET
    email_enabled: Union[Unset, bool] = UNSET
    project_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        filters: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item = filters_item_data.to_dict()

                filters.append(filters_item)

        name = self.name
        description = self.description
        checks: Union[Unset, List[str]] = UNSET
        if not isinstance(self.checks, Unset):
            checks = self.checks

        slack_enabled = self.slack_enabled
        email_enabled = self.email_enabled
        project_id = self.project_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filters is not UNSET:
            field_dict["filters"] = filters
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if checks is not UNSET:
            field_dict["checks"] = checks
        if slack_enabled is not UNSET:
            field_dict["slack_enabled"] = slack_enabled
        if email_enabled is not UNSET:
            field_dict["email_enabled"] = email_enabled
        if project_id is not UNSET:
            field_dict["project_id"] = project_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.filter_condition import FilterCondition

        d = src_dict.copy()
        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:
            filters_item = FilterCondition.from_dict(filters_item_data)

            filters.append(filters_item)

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        checks = cast(List[str], d.pop("checks", UNSET))

        slack_enabled = d.pop("slack_enabled", UNSET)

        email_enabled = d.pop("email_enabled", UNSET)

        project_id = d.pop("project_id", UNSET)

        datapoint_filter_update = cls(
            filters=filters,
            name=name,
            description=description,
            checks=checks,
            slack_enabled=slack_enabled,
            email_enabled=email_enabled,
            project_id=project_id,
        )

        datapoint_filter_update.additional_properties = d
        return datapoint_filter_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.filter_condition import FilterCondition


T = TypeVar("T", bound="SummaryDatapointSearch")


@_attrs_define
class SummaryDatapointSearch:
    """
    Attributes:
        project_id (Union[Unset, str]): Project ID
        mut_id (Union[Unset, str]): Model ID
        from_date (Union[Unset, datetime.datetime]): Earliest date Default: isoparse('2022-12-31T23:59:59.999999').
        to_date (Union[Unset, datetime.datetime]): Latest date
        filters (Union[Unset, List['FilterCondition']]): List of filter conditions to apply. Defaults to None (i.e., all
            datapoints).
        checks (Union[Unset, List[Any]]): List of checks to only flag issues for. Defaults to None (i.e., all checks).
        timezone (Union[Unset, str]): IANA timezone to use for date filtering/aggregation (e.g., 'America/New_York').
            Defaults to None (i.e., UTC).
        precision (Union[Unset, str]): Time precision for the summary. Valid values include ['day', 'hour', 'minute'].
            Defaults to 'day'. Default: 'day'.
        filter_group_id (Union[Unset, str]): Filter group ID to search with
        group_column (Union[Unset, str]): Column to group by for the summary. Overwrites the default time-based grouping
            for the summary.
    """

    project_id: Union[Unset, str] = UNSET
    mut_id: Union[Unset, str] = UNSET
    from_date: Union[Unset, datetime.datetime] = isoparse("2022-12-31T23:59:59.999999")
    to_date: Union[Unset, datetime.datetime] = UNSET
    filters: Union[Unset, List["FilterCondition"]] = UNSET
    checks: Union[Unset, List[Any]] = UNSET
    timezone: Union[Unset, str] = UNSET
    precision: Union[Unset, str] = "day"
    filter_group_id: Union[Unset, str] = UNSET
    group_column: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        project_id = self.project_id
        mut_id = self.mut_id
        from_date: Union[Unset, str] = UNSET
        if not isinstance(self.from_date, Unset):
            from_date = self.from_date.isoformat()

        to_date: Union[Unset, str] = UNSET
        if not isinstance(self.to_date, Unset):
            to_date = self.to_date.isoformat()

        filters: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item = filters_item_data.to_dict()

                filters.append(filters_item)

        checks: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.checks, Unset):
            checks = self.checks

        timezone = self.timezone
        precision = self.precision
        filter_group_id = self.filter_group_id
        group_column = self.group_column

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if project_id is not UNSET:
            field_dict["project_id"] = project_id
        if mut_id is not UNSET:
            field_dict["mut_id"] = mut_id
        if from_date is not UNSET:
            field_dict["from_date"] = from_date
        if to_date is not UNSET:
            field_dict["to_date"] = to_date
        if filters is not UNSET:
            field_dict["filters"] = filters
        if checks is not UNSET:
            field_dict["checks"] = checks
        if timezone is not UNSET:
            field_dict["timezone"] = timezone
        if precision is not UNSET:
            field_dict["precision"] = precision
        if filter_group_id is not UNSET:
            field_dict["filter_group_id"] = filter_group_id
        if group_column is not UNSET:
            field_dict["group_column"] = group_column

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.filter_condition import FilterCondition

        d = src_dict.copy()
        project_id = d.pop("project_id", UNSET)

        mut_id = d.pop("mut_id", UNSET)

        _from_date = d.pop("from_date", UNSET)
        from_date: Union[Unset, datetime.datetime]
        if isinstance(_from_date, Unset):
            from_date = UNSET
        else:
            from_date = isoparse(_from_date)

        _to_date = d.pop("to_date", UNSET)
        to_date: Union[Unset, datetime.datetime]
        if isinstance(_to_date, Unset):
            to_date = UNSET
        else:
            to_date = isoparse(_to_date)

        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:
            filters_item = FilterCondition.from_dict(filters_item_data)

            filters.append(filters_item)

        checks = cast(List[Any], d.pop("checks", UNSET))

        timezone = d.pop("timezone", UNSET)

        precision = d.pop("precision", UNSET)

        filter_group_id = d.pop("filter_group_id", UNSET)

        group_column = d.pop("group_column", UNSET)

        summary_datapoint_search = cls(
            project_id=project_id,
            mut_id=mut_id,
            from_date=from_date,
            to_date=to_date,
            filters=filters,
            checks=checks,
            timezone=timezone,
            precision=precision,
            filter_group_id=filter_group_id,
            group_column=group_column,
        )

        summary_datapoint_search.additional_properties = d
        return summary_datapoint_search

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
import json
import os
import warnings
from typing import Any, Dict, List, TypedDict, Union

import httpx
from pydantic import BaseModel as PydanticBaseModel

from okareo.checks import BaseCheck
from okareo_api_client import Client
from okareo_api_client.api.default import (
    add_model_to_group_v0_groups_group_id_models_post,
    check_create_or_update_v0_check_create_or_update_post,
    check_delete_v0_check_check_id_delete,
    check_generate_v0_check_generate_post,
    create_group_v0_groups_post,
    create_project_v0_projects_post,
    create_scenario_set_v0_scenario_sets_post,
    create_trace_eval_v0_groups_group_id_trace_eval_post,
    evaluate_v0_evaluate_post,
    find_test_data_points_v0_find_test_data_points_post,
    generate_scenario_set_v0_scenario_sets_generate_post,
    get_all_checks_v0_checks_get,
    get_all_projects_v0_projects_get,
    get_check_v0_check_check_id_get,
    get_datapoints_filter_v0_find_datapoints_filter_post,
    get_datapoints_v0_find_datapoints_post,
    get_scenario_set_data_points_v0_scenario_data_points_scenario_id_get,
    register_model_v0_register_model_post,
    scenario_sets_upload_v0_scenario_sets_upload_post,
)
from okareo_api_client.errors import UnexpectedStatus
from okareo_api_client.models.body_check_delete_v0_check_check_id_delete import (
    BodyCheckDeleteV0CheckCheckIdDelete,
)
from okareo_api_client.models.body_scenario_sets_upload_v0_scenario_sets_upload_post import (
    BodyScenarioSetsUploadV0ScenarioSetsUploadPost,
)
from okareo_api_client.models.check_create_update_schema import CheckCreateUpdateSchema
from okareo_api_client.models.check_create_update_schema_check_config import (
    CheckCreateUpdateSchemaCheckConfig,
)
from okareo_api_client.models.create_group_v0_groups_post_source import (
    CreateGroupV0GroupsPostSource,
)
from okareo_api_client.models.datapoint_filter_search import DatapointFilterSearch
from okareo_api_client.models.datapoint_list_item import DatapointListItem
from okareo_api_client.models.datapoint_search import DatapointSearch
from okareo_api_client.models.error_response import ErrorResponse
from okareo_api_client.models.evaluation_payload import EvaluationPayload
from okareo_api_client.models.evaluation_payload_metrics_kwargs import (
    EvaluationPayloadMetricsKwargs,
)
from okareo_api_client.models.evaluator_brief_response import EvaluatorBriefResponse
from okareo_api_client.models.evaluator_detailed_response import (
    EvaluatorDetailedResponse,
)
from okareo_api_client.models.evaluator_generate_response import (
    EvaluatorGenerateResponse,
)
from okareo_api_client.models.evaluator_spec_request import EvaluatorSpecRequest
from okareo_api_client.models.find_test_data_point_payload import (
    FindTestDataPointPayload,
)
from okareo_api_client.models.full_data_point_item import FullDataPointItem
from okareo_api_client.models.model_under_test_response import ModelUnderTestResponse
from okareo_api_client.models.model_under_test_schema import ModelUnderTestSchema
from okareo_api_client.models.project_response import ProjectResponse
from okareo_api_client.models.project_schema import ProjectSchema
from okareo_api_client.models.scenario_data_poin_response import (
    ScenarioDataPoinResponse,
)
from okareo_api_client.models.scenario_set_create import ScenarioSetCreate
from okareo_api_client.models.scenario_set_generate import ScenarioSetGenerate
from okareo_api_client.models.scenario_set_response import ScenarioSetResponse
from okareo_api_client.models.scenario_type import ScenarioType
from okareo_api_client.models.seed_data import SeedData
from okareo_api_client.models.test_data_point_item import TestDataPointItem
from okareo_api_client.models.test_run_item import TestRunItem
from okareo_api_client.models.test_run_type import TestRunType
from okareo_api_client.types import UNSET, File, Unset

from .common import BASE_URL, HTTPX_TIME_OUT
from .model_under_test import BaseModel, ModelUnderTest

CHECK_DEPRECATION_WARNING = (
    "The `evaluator` naming convention is deprecated and will not be supported in a future release. "
    "Please use `check` in place of `evaluator` when invoking this method."
)

# Error message for empty generation
# TODO: Determine warning criteria by status_code = 422
_EMPTY_GENERATION_MESSAGE = "No scenario rows generated for scenario_id"

CUSTOM_MODEL_STRS = ["custom", "custom_batch"]


def check_deprecation_warning() -> None:
    warnings.warn(CHECK_DEPRECATION_WARNING, DeprecationWarning, stacklevel=2)


class SeedDataRow(TypedDict):
    input: Union[dict, list, str]
    result: Union[dict, list, str]


class BaseGenerationSchema(PydanticBaseModel):
    """A base schema class for specifying structured outputs to synthetic data generators."""

    @classmethod
    def to_dict(cls) -> dict:
        return cls.model_json_schema()


class Okareo:
    """A class for interacting with Okareo API and for formatting request data."""

    def __init__(
        self, api_key: str, base_path: str = BASE_URL, timeout: float = HTTPX_TIME_OUT  # type: ignore
    ):
        self.api_key = api_key
        self.client = Client(
            base_url=base_path, raise_on_unexpected_status=True
        )  # otherwise everything except 201 and 422 is swallowed
        response = get_all_projects_v0_projects_get.sync(
            client=self.client,
            api_key=self.api_key,
        )
        self.validate_response(response)

    @staticmethod
    def seed_data_from_list(data_list: List[SeedDataRow]) -> List[SeedData]:
        """
        Create a list of SeedData objects from a list of dictionaries.

        Each dictionary in the input list must have 'input' and 'result' keys.

        Args:
            data_list (List[SeedDataRow]): A list of dictionaries, where each dictionary
                                        contains 'input' and 'result' keys.

        Returns:
            List[SeedData]: A list of SeedData objects created from the input dictionaries.
        """
        seed_data_list = []
        for data in data_list:
            seed_input: Union[dict, list, str] = data["input"]
            seed_result: Union[dict, list, str] = data["result"]
            seed_data = SeedData(input_=seed_input, result=seed_result)
            seed_data_list.append(seed_data)

        return seed_data_list

    def get_projects(self) -> List[ProjectResponse]:
        """
        Get a list of all Okareo projects available to the user.

        Returns:
            List[ProjectResponse]: A list of ProjectResponse objects accessible to the user.

        Raises:
            TypeError: If the API response is an error.
            ValueError: If no response is received from the API.
        """
        response = get_all_projects_v0_projects_get.sync(
            client=self.client,
            api_key=self.api_key,
        )
        self.validate_response(response)
        assert isinstance(response, List)

        return response

    def create_project(
        self, name: str, tags: Union[Unset, List[str]] = UNSET
    ) -> ProjectResponse:
        """
        Create a new Okareo project.

        Args:
            name (str): The name of the new project.
            tags (Union[Unset, List[str]], optional): Optional list of tags to associate with the project.

        Returns:
            ProjectResponse: The created ProjectResponse object.

        Raises:
            TypeError: If the API response is an error.
            ValueError: If no response is received from the API.
        """
        response = create_project_v0_projects_post.sync(
            client=self.client,
            api_key=self.api_key,
            json_body=ProjectSchema(name=name, tags=tags),
        )
        self.validate_response(response)
        assert isinstance(response, ProjectResponse)

        return response

    def _get_custom_model_invoker(
        self, data: Dict[str, Any]
    ) -> tuple[dict[str, Any], Any]:
        for custom_model_str in CUSTOM_MODEL_STRS:
            if custom_model_str in data["models"].keys():
                model_invoker = data["models"][custom_model_str]["model_invoker"]
                del data["models"][custom_model_str]["model_invoker"]
                return data, model_invoker
        if (
            "driver" in data["models"].keys()
            and data["models"]["driver"]["target"]["type"] == "custom_target"
        ):
            model_invoker = data["models"]["driver"]["target"]["model_invoker"]
            del data["models"]["driver"]["target"]["model_invoker"]
            return data, model_invoker
        return data, None

    def _set_custom_model_invoker(
        self, data: Dict[str, Any], model_invoker: Any
    ) -> Any:
        for custom_model_str in CUSTOM_MODEL_STRS:
            if custom_model_str in data.keys():
                data[custom_model_str]["model_invoker"] = model_invoker
        if "driver" in data.keys():
            data["driver"]["target"]["model_invoker"] = model_invoker
        return data

    def register_model(
        self,
        name: str,
        tags: Union[List[str], None] = None,
        project_id: Union[str, None] = None,
        model: Union[None, BaseModel, List[BaseModel]] = None,
        update: bool = False,
        sensitive_fields: Union[List[str], None] = None,
    ) -> ModelUnderTest:
        """
        Register a new Model Under Test (MUT) to use in an Okareo evaluation.

        Args:
            name (str): The name of the model. Model names must be unique within a project. Using the same name will return or update the existing model.
            tags (Union[List[str], None], optional): Optional list of tags to associate with the model.
            project_id (Union[str, None], optional): The project ID to associate the model with.
            model (Union[None, BaseModel, List[BaseModel]], optional): The model or list of models to register.
            update (bool, optional): Whether to update an existing model with the same name. Defaults to False.
            sensitive_fields (List[str], optional): A list of sensitive fields to mask in the model parameters. Defaults to None.

        Returns:
            ModelUnderTest: The registered ModelUnderTest object.

        Raises:
            TypeError: If the API response is an error.
            ValueError: If no response is received from the API.
        """
        if tags is None:
            tags = []
        data: Dict[str, Any] = {
            "name": name,
            "tags": tags,
            "update": update,
            "sensitive_fields": sensitive_fields,
        }
        # will rename name to model in the future api-breaking release
        model_invoker = None
        if isinstance(model, BaseModel) or (
            isinstance(model, list) and all(isinstance(x, BaseModel) for x in model)
        ):
            models = model if isinstance(model, list) else [model]
            data["models"] = {}
            for model in models:
                data["models"][model.type] = model.params()
            data, model_invoker = self._get_custom_model_invoker(data)
        if project_id is not None:
            data["project_id"] = project_id
        json_body = ModelUnderTestSchema.from_dict(data)
        response = register_model_v0_register_model_post.sync(
            client=self.client, api_key=self.api_key, json_body=json_body
        )

        self.validate_response(response)
        assert isinstance(response, ModelUnderTestResponse)
        model_data = data.get("models")
        if model_invoker and isinstance(model_data, dict):
            model_data = self._set_custom_model_invoker(model_data, model_invoker)
        if response.warning:
            print(response.warning)
        return ModelUnderTest(
            client=self.client,
            api_key=self.api_key,
            mut=response,
            models=model_data,
        )

    def create_scenario_set(
        self, create_request: ScenarioSetCreate
    ) -> ScenarioSetResponse:
        """
        Create a new scenario set to use in an Okareo evaluation or as a seed for synthetic data generation.

        Args:
            create_request (ScenarioSetCreate): The request object containing scenario set details and seed data. The ScenarioSetCreate object should include:

        Returns:
            ScenarioSetResponse: The created ScenarioSetResponse object.

        Raises:
            ValueError: If the seed data is empty or if no response is received from the API.
            TypeError: If the API response is an error.

        Example:
        ```python
        seed_data = okareo_client.seed_data_from_list([
            {"input": {"animal": "fish", "color": "red"}, "result": "red"},
            {"input": {"animal": "dog", "color": "blue"}, "result": "blue"},
            {"input": {"animal": "cat", "color": "green"}, "result": "green"}
        ])
        create_request = ScenarioSetCreate(name="My Scenario Set", seed_data=seed_data)
        okareo_client.create_scenario_set(create_request)
        ```
        """
        if create_request.seed_data == [] or create_request.seed_data is None:
            raise ValueError("Non-empty seed data is required to create a scenario set")

        response = create_scenario_set_v0_scenario_sets_post.sync(
            client=self.client, api_key=self.api_key, json_body=create_request
        )

        self.validate_response(response)
        assert isinstance(response, ScenarioSetResponse)
        if response.warning:
            print(response.warning)
        return response

    def upload_scenario_set(
        self, scenario_name: str, file_path: str, project_id: Union[Unset, str] = UNSET
    ) -> ScenarioSetResponse:
        """
        Upload a file as a scenario set to use in an Okareo evaluation or as a seed for synthetic data generation.

        Args:
            scenario_name (str): The name to assign to the uploaded scenario set.
            file_path (str): The path to the file to upload.
            project_id (Union[Unset, str], optional): The project ID to associate with the scenario set.

        Returns:
            ScenarioSetResponse: The created ScenarioSetResponse object.

        Raises:
            UnexpectedStatus: If the API returns an unexpected status.
            TypeError: If the API response is an error.
            ValueError: If no response is received from the API.

        Example:
        ```python
        project_id = "your_project_id"  # Optional, can be None
        okareo_client.upload_scenario_set(
            scenario_name="My Uploaded Scenario Set",
            file_path="/path/to/scenario_set_file.json",
            project_id=project_id or None,
        )
        ```
        """
        try:
            file_name = os.path.basename(file_path)

            with open(file_path, "rb") as binary_io:
                multipart_body = BodyScenarioSetsUploadV0ScenarioSetsUploadPost(
                    name=scenario_name,
                    project_id=project_id,
                    file=File(file_name=file_name, payload=binary_io),
                )
                response = scenario_sets_upload_v0_scenario_sets_upload_post.sync(
                    client=self.client,
                    api_key=self.api_key,
                    multipart_data=multipart_body,
                )

            self.validate_response(response)
            assert isinstance(response, ScenarioSetResponse)
            if response.warning:
                print(response.warning)
            return response
        except UnexpectedStatus as e:
            print(e.content)
            raise

    def download_scenario_set(
        self,
        scenario: Union[ScenarioSetResponse, str],
        file_path: str = "",
    ) -> Any:
        """
        Download a scenario set from Okareo to the client's local filesystem.

        Args:
            scenario_set (ScenarioSetResponse): The scenario set to download.
            file_path (str, optional): The path where the file will be saved. If not provided, uses scenario set name.

        Returns:
            File: The downloaded file object.

        Example:
        ```python
        response_file = okareo_client.download_scenario_set(create_scenario_set)
        with open(response_file.name) as scenario_file:
            for line in scenario_file:
                print(line)
        ```
        """
        try:
            scenario_id = (
                scenario if isinstance(scenario, str) else scenario.scenario_id
            )
            url = f"{BASE_URL}/v0/scenario_sets_download/{scenario_id}"
            headers = {
                "accept": "application/json",
                "api-key": self.api_key,
            }
            response = httpx.get(
                url,
                headers=headers,
            )
            filename = response.headers["content-disposition"].split('"')[1]
            if file_path != "":
                filename = file_path
            binary_file = open(filename, "wb")
            binary_file.write(response.content)
            binary_file.close()
            return binary_file
        except Exception as e:
            print(e)
            raise

    def generate_scenarios(
        self,
        source_scenario: Union[str, ScenarioSetResponse],
        name: str,
        number_examples: int,
        project_id: Union[Unset, str] = UNSET,
        generation_type: Union[Unset, ScenarioType] = ScenarioType.REPHRASE_INVARIANT,
    ) -> ScenarioSetResponse:
        """
        Generate a synthetic scenario set based on an existing seed scenario.

        Args:
            source_scenario (Union[str, ScenarioSetResponse]): The source scenario set or its ID to generate from.
            name (str): The name for the new generated scenario set.
            number_examples (int): The number of synthetic examples to generate per seed scenario row.
            project_id (Union[Unset, str], optional): The project ID to associate with the generated scenario set.
            generation_type (Union[Unset, ScenarioType], optional): The type of scenario generation to use.

        Returns:
            ScenarioSetResponse: The generated synthetic scenario set.

        Raises:
            TypeError: If the API response is an error.
            ValueError: If no response is received from the API.

        Example:
        ```python
        source_scenario = "source_scenario_id"  # or ScenarioSetResponse object
        generated_set = okareo_client.generate_scenarios(
            source_scenario=source_scenario,
            name="Generated Scenario Set",
            number_examples=100,
            project_id="your_project_id",
            generation_type=ScenarioType.REPHRASE_INVARIANT
        )
        print(generated_set.app_link) # Prints the link to the generated scenario set
        ```
        """
        scenario_id = (
            source_scenario.scenario_id
            if isinstance(source_scenario, ScenarioSetResponse)
            else source_scenario
        )
        return self.generate_scenario_set(
            ScenarioSetGenerate(
                source_scenario_id=scenario_id,
                name=name,
                number_examples=number_examples,
                project_id=project_id,
                generation_type=generation_type,
            )
        )

    def generate_scenario_set(
        self, create_request: ScenarioSetGenerate
    ) -> ScenarioSetResponse:
        """
        Generate a synthetic scenario set based on an existing seed scenario and a ScenarioSetGenerate object. Offers more controls than the comparable `generate_scenarios` method.

        Args:
            create_request (ScenarioSetGenerate): The request object specifying scenario generation parameters.

        Returns:
            ScenarioSetResponse: The generated synthetic scenario set.

        Example:
        ```python
        generate_request = ScenarioSetGenerate(
            source_scenario_id="seed_scenario_id",
            name="My Synthetic Scenario Set",
            number_examples=50,
            project_id="your_project_id",
            generation_type=ScenarioType.REPHRASE_INVARIANT,
        )
        generated_set = okareo_client.generate_scenario_set(generate_request)
        print(generated_set.app_link)  # Prints the link to the generated scenario set
        ```
        """
        response = generate_scenario_set_v0_scenario_sets_generate_post.sync(
            client=self.client, api_key=self.api_key, json_body=create_request
        )

        if self.validate_generate_scenario_response(response):
            self.validate_response(response)
            assert isinstance(response, ScenarioSetResponse)
        else:
            # return empty response
            return ScenarioSetResponse(
                project_id=(
                    create_request.project_id if create_request.project_id else ""
                ),
                scenario_id="",
                time_created=datetime.datetime.now(),
                type=(
                    create_request.generation_type.value
                    if create_request.generation_type
                    else ""
                ),
            )

        return response

    def get_scenario_data_points(
        self, scenario_id: str
    ) -> List[ScenarioDataPoinResponse]:
        """
        Fetch the scenario data points associated with a scenario set with scenario_id.

        Args:
            scenario_id (str): The ID of the scenario set to fetch data points for.

        Returns:
            List[ScenarioDataPoinResponse]: A list of scenario data point responses associated with the scenario set.

        Example:
        ```python
        okareo_client = Okareo(api_key="your_api_key")
        scenario_id = "your_scenario_id"
        data_points = okareo_client.get_scenario_data_points(scenario_id)
        for dp in data_points:
            print(dp.input_, dp.result)
        ```
        """
        response = (
            get_scenario_set_data_points_v0_scenario_data_points_scenario_id_get.sync(
                client=self.client, api_key=self.api_key, scenario_id=scenario_id
            )
        )

        self.validate_response(response)
        assert isinstance(response, List)

        return response

    def find_test_data_points(
        self, test_data_point_payload: FindTestDataPointPayload
    ) -> Union[List[Union[TestDataPointItem, FullDataPointItem]], ErrorResponse]:
        """
        Fetch the test run data points associated as specified in the payload.

        Args:
            test_data_point_payload (FindTestDataPointPayload): The payload specifying the test data point search criteria.

        Returns:
            Union[List[Union[TestDataPointItem, FullDataPointItem]], ErrorResponse]:
                A list of test or full data point items, or an error response.

        Example:
        ```python
        from okareo_api_client.models.find_test_data_point_payload import (
            FindTestDataPointPayload,
        )

        test_run_id = "your_test_run_id"  # Replace with your actual test run ID
        payload = FindTestDataPointPayload(
            test_run_id=test_run_id,
        )
        data_points = okareo_client.find_test_data_points(payload)
        for dp in data_points:
            print(dp)
        ```
        """
        data = find_test_data_points_v0_find_test_data_points_post.sync(
            client=self.client, api_key=self.api_key, json_body=test_data_point_payload
        )
        if not data:
            return []
        return data

    def validate_response(self, response: Any) -> None:
        if isinstance(response, ErrorResponse):
            error_message = f"error: {response.detail}"
            print(error_message)
            raise TypeError(error_message)
        if response is None:
            print("Received no response (None) from the API")
            raise ValueError("No response received")

    def validate_generate_scenario_response(self, response: Any) -> bool:
        if (
            isinstance(response, ErrorResponse)
            and _EMPTY_GENERATION_MESSAGE in response.detail[0]
        ):
            # TODO: parse 422 status code rather than string matching
            # Generated scenario is empty. Raise warning rather than error.
            warning_message = f"warning: {response.detail}"
            print(warning_message)
            warnings.warn(
                warning_message,
                category=UserWarning,
                stacklevel=2,
            )
            return False
        return True

    def find_datapoints(
        self, datapoint_search: DatapointSearch
    ) -> Union[List[DatapointListItem], ErrorResponse]:
        """
        Fetch the datapoints specified by a Datapoint Search.

        Args:
            datapoint_search (DatapointSearch): The search criteria for fetching datapoints.

        Returns:
            Union[List[DatapointListItem], ErrorResponse]: A list of datapoint items matching the search, or an error response.

        Example:
        ```python
        from okareo_api_client.models.datapoint_search import DatapointSearch

        ### Search based on a test run ID
        test_run__id = "your_test_run_id"  # Replace with your actual test run ID
        search = DatapointSearch(
            test_run_id=test_run__id,
        )
        datapoints = okareo_client.find_datapoints(search)
        for dp in datapoints:
            print(dp)

        ### Search based on a context token from a logger
        logger_config = {
            "api_key": "<API_KEY>",
            "tags": ["logger-test"],
            "context_token": random_string(10),
        }
        # Use the logger config to log completions from CrewAI or Autogen
        ...

        # Search for the logged datapoints by the context token
        search = DatapointSearch(
            context_token=context_token,
        )
        datapoints = okareo_client.find_datapoints(search)
        for dp in datapoints:
            print(dp)
        ```
        """
        data = get_datapoints_v0_find_datapoints_post.sync(
            client=self.client,
            api_key=self.api_key,
            json_body=datapoint_search,
        )
        if not data:
            return []
        return data

    def find_datapoints_filter(
        self, datapoint_search: DatapointFilterSearch
    ) -> Union[List[DatapointListItem], ErrorResponse]:
        """
        Fetch the datapoints specified by a Datapoint Search.

        Args:
            datapoint_search (DatapointSearch): The search criteria for fetching datapoints.

        Returns:
            Union[List[DatapointListItem], ErrorResponse]: A list of datapoint items matching the search, or an error response.

        Example:
        ```python
        from okareo_api_client.models.datapoint_search import DatapointFilterSearch

        ### Search based on a test run ID
        test_run__id = "your_test_run_id"  # Replace with your actual test run ID
        search = DatapointFilterSearch(
            test_run_id=test_run__id,
        )
        datapoints = okareo_client.find_datapoints(search)
        for dp in datapoints:
            print(dp)

        ### Find datapoints based on filters on datapoints fields
        from okareo_api_client.models.datapoint_filter_search import DatapointFilterSearch
        from okareo_api_client.models.filter_condition import FilterCondition
        from okareo_api_client.models.comparison_operator import ComparisonOperator

        search = DatapointFilterSearch(
            filters=[FilterCondition(
                field=DatapointField.TEST_RUN_ID,
                operator=ComparisonOperator.EQUAL,
                value="France"
            )]
        )
        datapoints = okareo_client.find_datapoints_filter(search)
        for dp in datapoints:
            print(dp)
        ```
        """
        data = get_datapoints_filter_v0_find_datapoints_filter_post.sync(
            client=self.client,
            api_key=self.api_key,
            json_body=datapoint_search,
        )
        if not data:
            return []
        return data

    def generate_evaluator(
        self, create_evaluator: EvaluatorSpecRequest
    ) -> EvaluatorGenerateResponse:
        check_deprecation_warning()
        return self.generate_check(create_evaluator)

    def generate_check(
        self, create_check: EvaluatorSpecRequest
    ) -> EvaluatorGenerateResponse:
        """
        Generate the contents of a Check based on an EvaluatorSpecRequest. Can be used to generate a behavioral (model-based) or a deterministic (code-based) check. Check names must be unique within a project.

        Args:
            create_check (EvaluatorSpecRequest): The specification for the check to generate.

        Returns:
            EvaluatorGenerateResponse: The generated check response.

        Example:
        ```python
        from okareo_api_client.models.evaluator_spec_request import EvaluatorSpecRequest
        from okareo.okareo import OkareoClient, BaseCheck

        # Generate a behavioral model-based check
        spec = EvaluatorSpecRequest(
            description="Checks if the output contains toxic language.",
            requires_scenario_input=False,
            requires_scenario_result=False,
            output_data_type="bool", # bool, int, float
        )
        okareo_client = Okareo(api_key="your_api_key")
        generated_check = okareo_client.generate_check(spec)

        # Inspect the generated check to ensure it meets your requirements
        print(generated_check)

        # Upload the generated check to Okareo to use in evaluations
        toxicity_check = okareo.create_or_update_check(
            name="toxicity_check",
            description=generated_check.description,
            check=ModelBasedCheck(  # type: ignore
                prompt_template=check.generated_prompt,
                check_type=CheckOutputType.PASS_FAIL,
            ),
        )
        # Inspect the uploaded check
        print(toxicity_check)
        ```
        """
        response = check_generate_v0_check_generate_post.sync(
            client=self.client, api_key=self.api_key, json_body=create_check
        )
        self.validate_response(response)
        assert isinstance(response, EvaluatorGenerateResponse)

        return response

    def get_all_evaluators(self) -> List[EvaluatorBriefResponse]:
        check_deprecation_warning()
        return self.get_all_checks()

    def get_all_checks(self) -> List[EvaluatorBriefResponse]:
        """
        Fetch all available checks.

        Args:
            None

        Returns:
            List[EvaluatorBriefResponse]: A list of EvaluatorBriefResponse objects representing all available checks.

        Example:
        ```python
        checks = okareo_client.get_all_checks()
        for check in checks:
            print(check.name, check.id)
        ```
        """
        response = get_all_checks_v0_checks_get.sync(
            client=self.client,
            api_key=self.api_key,
        )
        self.validate_response(response)
        assert isinstance(response, List)

        return response

    def get_evaluator(self, evaluator_id: str) -> EvaluatorDetailedResponse:
        check_deprecation_warning()
        return self.get_check(evaluator_id)

    def get_check(self, check_id: str) -> EvaluatorDetailedResponse:
        """
        Fetch details for a specific check.

        Args:
            check_id (str): The ID of the check to fetch.

        Returns:
            EvaluatorDetailedResponse: The detailed response for the specified check.

        Example:
        ```python
        check_id = "your_check_id"
        check_details = okareo_client.get_check(check_id)
        print(check_details)
        ```
        """
        response = get_check_v0_check_check_id_get.sync(
            client=self.client, api_key=self.api_key, check_id=check_id
        )
        self.validate_response(response)
        assert isinstance(response, EvaluatorDetailedResponse)

        return response

    def delete_evaluator(self, evaluator_id: str, evaluator_name: str) -> str:
        check_deprecation_warning()
        return self.delete_check(evaluator_id, evaluator_name)

    def delete_check(self, check_id: str, check_name: str) -> str:
        """
        Deletes a check identified by its ID and name.

        Args:
            check_id (str): The unique identifier of the check to delete.
            check_name (str): The name of the check to delete.

        Returns:
            str: A message indicating the result of the deletion.

        Example:
        ```python
        result = okareo_client.delete_check(check_id="abc123", check_name="MyCheck")
        print(result)  # Output: Check deletion was successful
        ```
        """
        check_delete_v0_check_check_id_delete.sync(
            client=self.client,
            api_key=self.api_key,
            check_id=check_id,
            form_data=BodyCheckDeleteV0CheckCheckIdDelete.from_dict(
                {"name": check_name}
            ),
        )
        return "Check deletion was successful"

    def create_or_update_check(
        self, name: str, description: str, check: BaseCheck
    ) -> EvaluatorDetailedResponse:
        """
        Create or update an existing check. If the check with 'name' already exists, then this method will update the existing check. Otherwise, this method will create a new check.

        Args:
            name (str): The unique name of the check to create or update.
            description (str): A human-readable description of the check.
            check (BaseCheck): An instance of BaseCheck containing the check configuration.

        Returns:
            EvaluatorDetailedResponse: The detailed response from the evaluator after creating or updating the check.

        Raises:
            AssertionError: If the response is not an instance of EvaluatorDetailedResponse.
            ValueError: If the response validation fails.

        Example:
        ```python
        from okareo.checks import CheckOutputType, ModelBasedCheck

        # Define your custom check; here we use a ModelBasedCheck as an example.
        # Mustache template is used in the prompt to pipe scenario input and generated output
        my_check = ModelBasedCheck(
            prompt_template="Only output the number of words in the following text: {scenario_input} {generation}",
            check_type=CheckOutputType.PASS_FAIL,
        )

        # Create or update the check
        response = okareo_client.create_or_update_check(
            name="my_word_count_check",
            description="Custom check for counting combined total number of words in input and output.",
            check=my_check
        )

        print(response)
        ```
        """
        check_config = CheckCreateUpdateSchemaCheckConfig.from_dict(
            check.check_config()
        )
        response = check_create_or_update_v0_check_create_or_update_post.sync(
            client=self.client,
            api_key=self.api_key,
            json_body=CheckCreateUpdateSchema(
                name=name, description=description, check_config=check_config
            ),
        )
        self.validate_response(response)
        assert isinstance(response, EvaluatorDetailedResponse)

        return response

    def create_group(
        self,
        name: str,
        tags: Union[List[str], None] = None,
        source: Union[dict, None] = None,
    ) -> Any:
        json_body = CreateGroupV0GroupsPostSource()
        if source:
            json_body.additional_properties.update(source)
        response = create_group_v0_groups_post.sync_detailed(
            client=self.client,
            json_body=json_body,
            name=name,
            tags=tags,
            api_key=self.api_key,
        )
        self.validate_response(response)

        return json.loads(response.content)

    def add_model_to_group(self, group: Any, model: Any) -> Any:
        response = add_model_to_group_v0_groups_group_id_models_post.sync_detailed(
            client=self.client,
            group_id=group.get("id", ""),
            model_id=model.mut_id,
            api_key=self.api_key,
        )
        self.validate_response(response)
        return response.parsed.additional_properties  # type: ignore

    def create_trace_eval(self, group: Any, context_token: str) -> Any:
        """
        Create a trace evaluation for a group.

        Args:
            group_id (str): The ID of the group.
            context_token (str): The context token for the trace.

        Returns:
            The created trace evaluation details.

        Raises:
            OkareoAPIException: If the API request fails.
        """
        response = create_trace_eval_v0_groups_group_id_trace_eval_post.sync_detailed(
            client=self.client,
            group_id=group.get("id", ""),
            context_token=context_token,
            api_key=self.api_key,
        )
        self.validate_response(response)
        return response.parsed

    def evaluate(
        self,
        name: str,
        test_run_type: TestRunType,
        scenario_id: Union[Unset, str] = UNSET,
        datapoint_ids: Union[Unset, list[str]] = UNSET,
        filter_group_id: Union[Unset, str] = UNSET,
        tags: Union[Unset, list[str]] = UNSET,
        metrics_kwargs: Union[Dict[str, Any], Unset] = UNSET,
        checks: Union[Unset, list[str]] = UNSET,
    ) -> TestRunItem:
        """
        Evaluate datapoints using the specified parameters.

        Args:
            scenario_id: ID of the scenario set
            metrics_kwargs: Dictionary of metrics to be measured
            name: Name of the test run
            test_run_type: Type of test run
            tags: Tags for filtering test runs
            checks: List of checks to include
            datapoint_ids: List of datapoint IDs to filter by
            filter_group_id: ID of the datapoint filter group to apply

        Returns:
            TestRunItem: The evaluation results as a TestRunItem object.

        Example:
        ```python
        checks = ["model_refusal"]  # one or more checks to apply in the evaluation
        test_run = okareo.evaluate(
            name="My Test Run",
            test_run_type=TestRunType.NL_GENERATION,
            checks=checks,
            datapoint_ids=["datapoint_id_1", "datapoint_id_2"],
        )
        print(test_run.app_link)  # View link to eval results in Okareo app
        ```
        """
        payload = EvaluationPayload(
            metrics_kwargs=EvaluationPayloadMetricsKwargs.from_dict(
                metrics_kwargs or {}
            ),
            name=name,
            type=test_run_type,
            tags=tags,
            checks=checks,
            scenario_id=scenario_id,
            datapoint_ids=datapoint_ids,
            filter_group_id=filter_group_id,
        )

        response = evaluate_v0_evaluate_post.sync(
            client=self.client,
            json_body=payload,
            api_key=self.api_key,
        )
        self.validate_response(response)
        if isinstance(response, ErrorResponse):
            error_message = f"error: {response}, {response.detail}"
            print(error_message)
            raise
        if not response:
            print("Empty response from API")
        assert response is not None
        return response



from okareo.checks import CodeBasedCheck


class Check(CodeBasedCheck):
    @staticmethod
    def evaluate(model_output: str, scenario_input: str, scenario_result: str, metadata: dict, model_input: str) -> int:  # fmt: skip
        return len(model_input)



import os
import random
import string
from typing import Any, Callable

import pytest

from okareo.common import BASE_URL

API_KEY = os.environ.get("OKAREO_API_KEY", "no-api-key")


class OkareoAPIhost:
    def __init__(self, path: str, is_mock: bool) -> None:
        self.path = path
        self.is_mock = is_mock


def integration(func: Callable) -> Any:
    params = [
        OkareoAPIhost("http://mocked.com", True),
        OkareoAPIhost(BASE_URL, False),  # type: ignore
    ]
    return pytest.mark.parametrize("okareo_api", params)(func)


def random_string(length: int) -> str:
    return "".join(random.choices(string.ascii_letters, k=length))


# if unavailable, mock a valid OpenAI API kley
OPENAI_API_KEY = os.environ.get("OPENAI_API_KEY", f"sk-{random_string(48)}")



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.test_custom_endpoint_request_end_session_params import TestCustomEndpointRequestEndSessionParams
    from ..models.test_custom_endpoint_request_next_message_params import TestCustomEndpointRequestNextMessageParams
    from ..models.test_custom_endpoint_request_start_session_params import TestCustomEndpointRequestStartSessionParams


T = TypeVar("T", bound="TestCustomEndpointRequest")


@_attrs_define
class TestCustomEndpointRequest:
    """
    Attributes:
        next_message_params (TestCustomEndpointRequestNextMessageParams): API parameters to get the next message in the
            session.
        start_session_params (Union[Unset, TestCustomEndpointRequestStartSessionParams]): API parameters to start a
            session. Optional
        end_session_params (Union[Unset, TestCustomEndpointRequestEndSessionParams]): API parameters to end a session.
            Optional.
        mut_id (Union[Unset, str]): ID of the model to use for the custom endpoint. Optional.
        sensitive_fields (Union[Unset, List[str]]): List of sensitive fields to redact in the response. Optional.
    """

    next_message_params: "TestCustomEndpointRequestNextMessageParams"
    start_session_params: Union[Unset, "TestCustomEndpointRequestStartSessionParams"] = UNSET
    end_session_params: Union[Unset, "TestCustomEndpointRequestEndSessionParams"] = UNSET
    mut_id: Union[Unset, str] = UNSET
    sensitive_fields: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        next_message_params = self.next_message_params.to_dict()

        start_session_params: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.start_session_params, Unset):
            start_session_params = self.start_session_params.to_dict()

        end_session_params: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.end_session_params, Unset):
            end_session_params = self.end_session_params.to_dict()

        mut_id = self.mut_id
        sensitive_fields: Union[Unset, List[str]] = UNSET
        if not isinstance(self.sensitive_fields, Unset):
            sensitive_fields = self.sensitive_fields

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "next_message_params": next_message_params,
            }
        )
        if start_session_params is not UNSET:
            field_dict["start_session_params"] = start_session_params
        if end_session_params is not UNSET:
            field_dict["end_session_params"] = end_session_params
        if mut_id is not UNSET:
            field_dict["mut_id"] = mut_id
        if sensitive_fields is not UNSET:
            field_dict["sensitive_fields"] = sensitive_fields

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.test_custom_endpoint_request_end_session_params import TestCustomEndpointRequestEndSessionParams
        from ..models.test_custom_endpoint_request_next_message_params import TestCustomEndpointRequestNextMessageParams
        from ..models.test_custom_endpoint_request_start_session_params import (
            TestCustomEndpointRequestStartSessionParams,
        )

        d = src_dict.copy()
        next_message_params = TestCustomEndpointRequestNextMessageParams.from_dict(d.pop("next_message_params"))

        _start_session_params = d.pop("start_session_params", UNSET)
        start_session_params: Union[Unset, TestCustomEndpointRequestStartSessionParams]
        if isinstance(_start_session_params, Unset):
            start_session_params = UNSET
        else:
            start_session_params = TestCustomEndpointRequestStartSessionParams.from_dict(_start_session_params)

        _end_session_params = d.pop("end_session_params", UNSET)
        end_session_params: Union[Unset, TestCustomEndpointRequestEndSessionParams]
        if isinstance(_end_session_params, Unset):
            end_session_params = UNSET
        else:
            end_session_params = TestCustomEndpointRequestEndSessionParams.from_dict(_end_session_params)

        mut_id = d.pop("mut_id", UNSET)

        sensitive_fields = cast(List[str], d.pop("sensitive_fields", UNSET))

        test_custom_endpoint_request = cls(
            next_message_params=next_message_params,
            start_session_params=start_session_params,
            end_session_params=end_session_params,
            mut_id=mut_id,
            sensitive_fields=sensitive_fields,
        )

        test_custom_endpoint_request.additional_properties = d
        return test_custom_endpoint_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
